  package com.mediamain.sdk_flutter

import android.app.Activity
import android.content.Context
import androidx.annotation.NonNull
import com.mediamain.sdk_flutter.util.*
import com.mediamain.sdk_flutter.util.PermissionUtils.Companion.requestPermissions
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry
import java.io.File
import java.util.*

  /** SdkFlutterPlugin */
class SdkFlutterPlugin : FlutterPlugin, MethodCallHandler, ActivityAware, PluginRegistry.RequestPermissionsResultListener {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var channel: MethodChannel
    private lateinit var mActivity: Activity
    private lateinit var mContext: Context

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "sdk_flutter")
        channel.setMethodCallHandler(this)

        mContext = flutterPluginBinding.applicationContext
//    oaidInit(flutterPluginBinding.applicationContext)
        LogUtils.e("===OAID===" + SDKOaidUtils.getOAID(flutterPluginBinding.applicationContext))
        LogUtils.e("===Nonce===" + ParamsUtils.getNonce())
        LogUtils.e("===Timestamp===" + ParamsUtils.getTimestamp())
    }

    override fun onDetachedFromEngine(binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            "version" ->
                result.success("Android ${android.os.Build.VERSION.RELEASE}")
            "getImei" ->
                result.success(ImeiPlugin.getImei(mActivity))
            "getDeviceId" ->
                result.success(ImeiPlugin.getImei(mActivity))
            "getOaid" ->
                result.success(ParamsUtils.getOaid(mContext))
            "getMD" -> {
                val sdkVersion: String? = call.argument<String>("sdkVersion")
                result.success(ParamsUtils.getMD(mActivity,sdkVersion))
            }
            "getNonce" ->
                result.success(ParamsUtils.getNonce())
            "getTcid" ->
                result.success(ParamsUtils.getTcid())
            "getTimestamp" ->
                result.success(ParamsUtils.getTimestamp())
            "getSignature" -> {
                val appSecret: String? = call.argument<String>("appSecret")
                val nonce: String? = call.argument<String>("nonce")
                val timestamp: String? = call.argument<String>("timestamp")
                val sdkVersion: String? = call.argument<String>("sdkVersion")
                result.success(ParamsUtils.getSignature(mActivity, appSecret, nonce, timestamp,sdkVersion))
                LogUtils.e("===appSecret===" + appSecret)
            }
            /*----------------------------------------------*/
            "getPackageName" ->
                result.success(ParamsUtils.getPackageName(mContext))
            "getAppName" ->
                result.success(ParamsUtils.getAppName(mContext))
            "getGPSLongitude" ->
                result.success(ParamsUtils.getGPSLongitude(mContext))
            "getGPSLatitude" ->
                result.success(ParamsUtils.getGPSLatitude(mContext))
            "getOSType" ->
                result.success(ParamsUtils.getOSType())
            "getNetType" ->
                result.success(ParamsUtils.getNetType(mContext))
            "getAppList" ->
                result.success(ParamsUtils.getAppList(mContext))
            "getActiveList" ->
                result.success(ParamsUtils.getActiveList(mContext))
            "getOSVersion" ->
                result.success(ParamsUtils.getOSVersion())
            "getPhoneBrand" ->
                result.success(ParamsUtils.getPhoneBrand())
            "getPhoneModel" ->
                result.success(ParamsUtils.getPhoneModel())
            "getPhoneElectric" ->
                result.success(ParamsUtils.getPhoneElectric(mContext))
            "getPhoneIp" ->
                result.success(ParamsUtils.getPhoneIp())
            "getNetCarrier" ->
                result.success(ParamsUtils.getNetCarrier(mContext))
            "getDevelopType" ->
                result.success(ParamsUtils.getDevelopType())
            "getSign" ->{
                val maps: Map<String, Any>? = call.argument<Map<String, Any>>("maps")
                val hostKey: String? = call.argument<String>("hostKey")
                LogUtils.e("Map ===" + maps.toString())
                LogUtils.e("hostKey ===" + hostKey)
                result.success(ParamsUtils.getSign(maps,hostKey))
            }
            "getAmassMd" ->{
                val appKey: String? = call.argument<String>("appKey")
                val unCollect: String? = call.argument<String>("unCollect")
                val sdkVersion: String? = call.argument<String>("sdkVersion")

                result.success(ParamsUtils.getAmass(mActivity,unCollect.toString(),appKey,sdkVersion))
            }
            "getSha1" ->{
                val maps: Map<String, Any>? = call.argument<Map<String, Any>>("sha1Map")

                val mMap: TreeMap<String, String> = TreeMap()
                mMap["appKey"] = maps?.get("appKey").toString()
                mMap["phoneBrand"] = maps?.get("phoneBrand").toString()
                mMap["phoneModel"] = maps?.get("phoneModel").toString()
                mMap["imei"] = maps?.get("imei").toString()
                mMap["oaid"] = maps?.get("oaid").toString()
                mMap["idfa"] = ""
                mMap["deviceId"] = maps?.get("deviceId").toString()
                mMap["sdkVersion"] = maps?.get("sdkVersion").toString()
                mMap["osType"] = maps?.get("osType").toString()
                mMap["patchVersion"] = "0.0"
                mMap["patchCode"] = "0"

                // 拼接要签名的参数字符串
                var signSB = StringBuilder()
                // 对map的key进行排序
                val keySet: Set<String> = mMap.keys
                val iter = keySet.iterator()
                while (iter.hasNext()) {
                    val key = iter.next()
                    signSB = signSB.append(key).append("=").append(maps?.get(key)).append("&")
                }
                val str = signSB.substring(0, signSB.length - 1);
                LogUtils.e("signSB ===" + str)
                result.success(ParamsUtils.getSha1(str))
            }
            "install" ->{
              val path : String? = call.argument<String>("path")
              val file = File(path)
              result.success(JCommonUtils.openFile(mContext, file))
            }
            "installBroadCast" ->{
                val path : String? = call.argument<String>("apkPath")
                val packageName : String? = call.argument<String>("packageName")
                val progress : String? = call.argument<String>("progress")
                val tuiaId : String? = call.argument<String>("tuiaId")
                result.success(DownloadUtils.getInstance().showNotification(mContext, path, packageName, "",
                    progress, false, tuiaId, true))
            }
            "registerBroadCast" ->{
                val path : String? = call.argument<String>("apkPath")
                val packageName : String? = call.argument<String>("packageName")
                val tuiaId : String? = call.argument<String>("tuiaId")
                result.success(DownloadUtils.getInstance().registerBroadCast(path, packageName, tuiaId))
            }
            "getIDFA" ->
                result.success("")
            "getCrashData" ->{
                val dataFrom = 0
                val configData = ""
                val dataType: String? = call.argument<String>("dataType")
                val appKey: String? = call.argument<String>("appKey")
                val sdkVersion: String? = call.argument<String>("sdkVersion")
                val crashLog: String? = call.argument<String>("crashLog")
                val crashType: String? = call.argument<String>("crashType")

                result.success(ParamsUtils.getCrashData(mActivity,dataFrom,configData,appKey,crashLog,dataType,true,sdkVersion,crashType))
            }
            else -> {
                result.notImplemented()
            }
        }
    }

    override fun onDetachedFromActivity() {
        LogUtils.e("===onDetachedFromActivity===")
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
        LogUtils.e("===onReattachedToActivityForConfigChanges===")
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        mActivity = binding.activity
        LogUtils.e("===onAttachedToActivity===  activity = " + mActivity)
        //权限
        requestPermissions(mActivity, ImeiPlugin.MY_PERMISSIONS_REQUEST_READ_PHONE_STATE)

//        LogUtils.e("===MD===" + ParamsUtils.getMD(mActivity))
//        LogUtils.e("===Imei===" + ParamsUtils.getImei(mActivity))
    }

    override fun onDetachedFromActivityForConfigChanges() {
        LogUtils.e("===onDetachedFromActivityForConfigChanges===")
    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<out String>?, grantResults: IntArray?): Boolean {
        LogUtils.e("===权限=== requestCode = ${requestCode}  permissions = ${permissions}")
        return false
    }
}
