package com.mediamain.sdk_flutter.util

import android.Manifest
import android.Manifest.permission.INTERNET
import android.annotation.SuppressLint
import android.app.Application
import android.content.Context
import android.content.Context.BATTERY_SERVICE
import android.content.pm.ApplicationInfo
import android.content.pm.PackageInfo
import android.content.pm.PackageManager
import android.location.Location
import android.location.LocationManager
import android.os.BatteryManager
import android.os.Build
import android.telephony.TelephonyManager
import android.text.TextUtils
import androidx.annotation.RequiresPermission
import com.mediamain.sdk_flutter.model.AppInfoBean
import java.net.InetAddress
import java.net.NetworkInterface
import java.nio.charset.Charset
import java.security.Key
import java.security.KeyFactory
import java.security.spec.X509EncodedKeySpec
import java.util.*
import javax.crypto.Cipher
import kotlin.collections.ArrayList


class CommonUtils {

    companion object{

        @SuppressLint("PrivateApi")
        fun getApplication(): Context? {
            try {
                val activityThread = Class.forName("android.app.ActivityThread")
                val thread : Any? = activityThread.getMethod("currentActivityThread").invoke(null)
                val app : Any? = activityThread.getMethod("getApplication").invoke(thread)
                return app as Application
            } catch (e : Exception) {
            }
            return null
        }

        /**
         * 统一判空
         */
        fun isEmpty(str: String?) : Boolean{
            if(str.toString() == "null"){
                return true
            }
            return TextUtils.isEmpty(str)

        }

        fun isEmptyList(list: List<*>?): Boolean {
            return null == list || list.isEmpty()
        }

        fun getGPSInfo(context: Context): Location? {
            var bestLocation: Location? = null
            try {
                if(!PermissionUtils.isGranted(Manifest.permission.ACCESS_FINE_LOCATION) && !PermissionUtils.isGranted(Manifest.permission.ACCESS_COARSE_LOCATION)){
                    return bestLocation
                }
                val lm = context.getSystemService(Context.LOCATION_SERVICE) as LocationManager
                //判断GPS是否正常启动
                if (lm.isProviderEnabled(LocationManager.GPS_PROVIDER) || lm.isProviderEnabled(LocationManager.NETWORK_PROVIDER)) {
                    try {
                        val providers = lm.getProviders(true)
                        for (provider in providers) {
                            val l = lm.getLastKnownLocation(provider) ?: continue
                            if (bestLocation == null || l.accuracy < bestLocation.accuracy) {
                                bestLocation = l
                            }
                        }
                    } catch (e: Exception) {
                    }
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
            return bestLocation
        }

        /**
         * 加密RSA
         * 用公钥加密
         *
         * @param data
         * @param key
         * @return
         */
        fun encryptRSA(data: String?, key: String?): String? {
            try {
                if (isEmpty(data) || isEmpty(key)) {
                    return ""
                }
                // 对公钥解密
                val keyBytes: ByteArray = JEncodeUtils.base64Decode(key?.toByteArray(charset("UTF-8")))
                // 取得公钥
                val x509KeySpec = X509EncodedKeySpec(keyBytes)
                val keyFactory: KeyFactory = KeyFactory.getInstance("RSA")
                val publicKey: Key = keyFactory.generatePublic(x509KeySpec)
                // 对数据加密
                val cipher: Cipher = Cipher.getInstance("RSA/ECB/PKCS1Padding")
                cipher.init(Cipher.ENCRYPT_MODE, publicKey)
                val inputArray = data?.toByteArray(charset("UTF-8"))
                val inputLength = inputArray?.size
                // 最大加密字节数，超出最大字节数需要分组加密
                val MAX_ENCRYPT_BLOCK = 117
                // 标识
                var offSet = 0
                var resultBytes = byteArrayOf()
                var cache: ByteArray
                if (inputLength != null) {
                    while (inputLength - offSet > 0) {
                        if (inputLength - offSet > MAX_ENCRYPT_BLOCK) {
                            cache = cipher.doFinal(inputArray, offSet, MAX_ENCRYPT_BLOCK)
                            offSet += MAX_ENCRYPT_BLOCK
                        } else {
                            cache = cipher.doFinal(inputArray, offSet, inputLength - offSet)
                            offSet = inputLength
                        }
                        resultBytes = Arrays.copyOf(resultBytes, resultBytes.size + cache.size)
                        System.arraycopy(cache, 0, resultBytes, resultBytes.size - cache.size, cache.size)
                    }
                }
                return String(JEncodeUtils.base64Encode(resultBytes), Charset.forName("UTF-8") )
            } catch (e: java.lang.Exception) {
                e.printStackTrace()
            }
            return ""
        }



        fun getAppName(packageName: String?,context: Context): String? {
            return if (isSpace(packageName)) "" else try {
                val pm: PackageManager = context.packageManager
                val pi = pm.getPackageInfo(packageName!!, 0)
                pi?.applicationInfo?.loadLabel(pm)?.toString()
            } catch (e: PackageManager.NameNotFoundException) {
                e.printStackTrace()
                ""
            }
        }

        /*--------------------------AppList-Start-----------------------------*/
        fun getAppList(context: Context): String? {
            var strWithDComma = ""
            try {
                val sb = StringBuilder()
                val appsInfo: List<AppInfoBean>? = getAppsInfo(context)
                if (appsInfo != null) {
                    for (appInfo in appsInfo) {
                        if (!appInfo.isSystem) {
                            sb.append(appInfo.packageName)
                            sb.append(",")
                        }
                    }
                }
                strWithDComma = sb.toString()
                if (strWithDComma.endsWith(",")) {
                    return strWithDComma.substring(0, strWithDComma.lastIndexOf(","))
                }
            } catch (e: java.lang.Exception) {
            }
            return strWithDComma
        }

        fun getAppsInfo(context: Context): List<AppInfoBean>? {
            val list: MutableList<AppInfoBean> = ArrayList()
            try {
                val pm: PackageManager = context.packageManager
                val installedPackages: List<PackageInfo> = pm.getInstalledPackages(0)
                for (pi in installedPackages) {
                    if (pi != null) {
                        val isSystem = ApplicationInfo.FLAG_SYSTEM and pi.applicationInfo.flags !== 0
                        if (!isSystem) {
                            val ai: AppInfoBean = getBean(pm, pi) ?: continue
                            list.add(ai)
                        }
                    }
                }
            } catch (e: java.lang.Exception) {
            }
            return list
        }

        private fun getBean(pm: PackageManager?, pi: PackageInfo?): AppInfoBean? {
            try {
                if (pm == null || pi == null) {
                    return null
                }
                val ai = pi.applicationInfo
                val packageName = pi.packageName
                val name = ai.loadLabel(pm).toString()
                val packagePath = ai.sourceDir
                val versionName = pi.versionName
                val versionCode = pi.versionCode
                val isSystem = ApplicationInfo.FLAG_SYSTEM and ai.flags != 0
                return AppInfoBean(packageName, name, packagePath, versionName, versionCode, isSystem)
            } catch (e: java.lang.Exception) {
            }
            return null
        }
        /*--------------------------AppList-End-----------------------------*/
        //电量
        fun getBatteryInfo(context: Context): Int {
            var intProperty = 0
            try {
                intProperty = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                    val manager: BatteryManager = context.getSystemService(BATTERY_SERVICE) as BatteryManager
                    manager.getIntProperty(BatteryManager.BATTERY_PROPERTY_CAPACITY)
                } else {
                    0
                }
            } catch (e: java.lang.Exception) {
            }
            return intProperty
        }

        @RequiresPermission(INTERNET)
        fun getIPAddress(useIPv4: Boolean): String? {
            try {
                val nis: Enumeration<NetworkInterface> = NetworkInterface.getNetworkInterfaces()
                val adds: LinkedList<InetAddress> = LinkedList()
                while (nis.hasMoreElements()) {
                    val ni: NetworkInterface = nis.nextElement()
                    // To prevent phone of xiaomi return "10.0.2.15"
                    if (!ni.isUp() || ni.isLoopback()) continue
                    val addresses: Enumeration<InetAddress> = ni.getInetAddresses()
                    while (addresses.hasMoreElements()) {
                        adds.addFirst(addresses.nextElement())
                    }
                }
                for (add in adds) {
                    if (!add.isLoopbackAddress()) {
                        val hostAddress: String = add.getHostAddress()
                        val isIPv4 = hostAddress.indexOf(':') < 0
                        if (useIPv4) {
                            if (isIPv4) return hostAddress
                        } else {
                            if (!isIPv4) {
                                val index = hostAddress.indexOf('%')
                                return if (index < 0) hostAddress.toUpperCase(Locale.getDefault()) else hostAddress.substring(0, index).toUpperCase(Locale.getDefault())
                            }
                        }
                    }
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
            return ""
        }

        //运营商
        fun getSimOperatorByMnc(context: Context): String? {
            val tm: TelephonyManager = (context.getSystemService(Context.TELEPHONY_SERVICE) as TelephonyManager)
            val operator: String = tm.simOperator ?: return ""
            return when (operator) {
                "46000", "46002", "46007", "46020" -> "中国移动"
                "46001", "46006", "46009" -> "中国联通"
                "46003", "46005", "46011" -> "中国电信"
                else -> operator
            }
        }

        /*-----------------------------------------------Private---------------------------------------------------*/
        private fun isSpace(s: String?): Boolean {
            if (s == null) return true
            var i = 0
            val len = s.length
            while (i < len) {
                if (!Character.isWhitespace(s[i])) {
                    return false
                }
                ++i
            }
            return true
        }
    }

}