package com.mediamain.sdk_flutter.util;

import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.os.Build;
import android.util.Log;

import com.mediamain.sdk_flutter.R;
import com.mediamain.sdk_flutter.broadcast.DownloadBroadCast;
import com.mediamain.sdk_flutter.broadcast.NotificationBroadCast;

import java.io.File;

import androidx.core.app.NotificationCompat;
import androidx.core.app.NotificationManagerCompat;
import androidx.core.content.ContextCompat;
import androidx.core.content.FileProvider;

/**
 * <p> File description: <p>
 * <p> Creator: Adroll   <p>
 * <p> Created date: 5/28/21 <p>
 * * * * * * * * * * * * * * * * * * * * * *
 * Thinking is more important than coding *
 * * * * * * * * * * * * * * * * * * * * * *
 */
public class DownloadUtils {

    private static volatile DownloadUtils INSTANCE;
    private static DownloadBroadCast downloadBroadCast;

    /**
     * 当前下载状态
     */
    public enum DownloadStatus {
        // 未下载
        UN_DOWNLOAD,
        // 正在下载
        DOWNLOADING,
        // 已下载未安装
        UN_INSTALL,
        // 已安装
        INSTALLED,
        // 暂停下载
        STOP_DOWNLOAD,
        // 下载失败
        DOWNLOAD_FAIL,
    }

    public static DownloadUtils getInstance(){
        if (INSTANCE == null){
            synchronized (DownloadUtils.class){
                if (INSTANCE == null){
                    INSTANCE = new DownloadUtils();
                }
            }
        }
        return INSTANCE;
    }

    public NotificationManagerCompat showNotification(Context context, String path, String packageName, String icon, String progressTemp,
                                                      boolean isCancelTask, String tuiaId, boolean isShowNotification){
        return createNotification(context, path, packageName, icon, progressTemp, isCancelTask, tuiaId, isShowNotification);
    }

    private NotificationManagerCompat createNotification(Context context, String path, String packageName, String icon, String progressTemp,
                                                         boolean isCancelTask, String tuiaId, boolean isShowNotification) {
        try {
            String channelId = "channelId";
            CharSequence channelName = "channelName";
            String channelDescription = "channelDescription";
            int progress = Integer.parseInt(progressTemp);

            int channelImportance = NotificationManager.IMPORTANCE_HIGH;
            int iconTemp = com.mediamain.sdk_flutter.R.drawable.fox_notification_download;

            // 创建通知栏广播
            Intent intent = new Intent(context, NotificationBroadCast.class);
            intent.putExtra("apkPath", path);
            intent.putExtra("packageName", packageName);
            intent.putExtra("icon", icon);
            intent.putExtra("tuiaId", tuiaId);
            intent.putExtra("progress", progress);
            PendingIntent pendingIntent = PendingIntent.getBroadcast(context.getApplicationContext(),
                    path.hashCode(), intent, PendingIntent.FLAG_UPDATE_CURRENT);

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                NotificationChannel notificationChannel = new NotificationChannel(channelId, channelName, channelImportance);
                // 设置描述 最长30字符
                notificationChannel.setDescription(channelDescription);
                // 该渠道的通知是否使用震动
                notificationChannel.enableLights(false);
                notificationChannel.enableVibration(false);
                notificationChannel.setVibrationPattern(new long[]{0});
                notificationChannel.setSound(null, null);
                // 设置显示模式
                notificationChannel.setLockscreenVisibility(NotificationCompat.VISIBILITY_PUBLIC);

                NotificationManager notificationManager = (NotificationManager) context.getSystemService(Context.NOTIFICATION_SERVICE);
                notificationManager.createNotificationChannel(notificationChannel);
            }

            NotificationCompat.Builder notificationCompatBuilder = new NotificationCompat.Builder(context.getApplicationContext(), channelId);
            Notification notification;
            if (progress < 100) {
                notification = notificationCompatBuilder
                        // Title for API <16 (4.0 and below) devices.
                        .setContentTitle(isCancelTask ? "暂停下载" : "正在下载")
                        .setSmallIcon(iconTemp)
                        .setLargeIcon(BitmapFactory.decodeResource(context.getResources(), iconTemp))
                        .setDefaults(NotificationCompat.DEFAULT_ALL)
                        .setColor(ContextCompat.getColor(context.getApplicationContext(), R.color.colorPrimary))
                        .setCategory(Notification.CATEGORY_REMINDER)
                        .setPriority(NotificationCompat.PRIORITY_MAX)
                        .setContentIntent(pendingIntent)
                        .setVibrate(new long[]{0})
                        .setSound(null)
                        .setProgress(100, progress, false)
                        .build();
            } else if (progress == 100){
                notification = notificationCompatBuilder
                        // Title for API <16 (4.0 and below) devices.
                        .setContentTitle("立即安装")
                        // Content for API <24 (7.0 and below) devices.
                        .setContentText("点击领取福利")
                        .setSmallIcon(iconTemp)
                        .setLargeIcon(BitmapFactory.decodeResource(context.getResources(), iconTemp))
                        .setDefaults(NotificationCompat.DEFAULT_ALL)
                        .setColor(ContextCompat.getColor(context.getApplicationContext(), R.color.colorPrimary))
                        .setCategory(Notification.CATEGORY_REMINDER)
                        .setPriority(NotificationCompat.PRIORITY_MAX)
                        .setContentIntent(pendingIntent)
                        .setVibrate(new long[]{0})
                        .setSound(null)
                        .build();
            } else {
                notification = notificationCompatBuilder
                        // Title for API <16 (4.0 and below) devices.
                        .setContentTitle("立即打开")
                        // Content for API <24 (7.0 and below) devices.
                        .setContentText("点击领取福利")
                        .setSmallIcon(iconTemp)
                        .setLargeIcon(BitmapFactory.decodeResource(context.getResources(), iconTemp))
                        .setDefaults(NotificationCompat.DEFAULT_ALL)
                        .setColor(ContextCompat.getColor(context.getApplicationContext(), R.color.colorPrimary))
                        .setCategory(Notification.CATEGORY_REMINDER)
                        .setPriority(NotificationCompat.PRIORITY_MAX)
                        .setContentIntent(pendingIntent)
                        .setVibrate(new long[]{0})
                        .setSound(null)
                        .build();
            }

            NotificationManagerCompat notificationManagerCompat = NotificationManagerCompat.from(context.getApplicationContext());

            if (isShowNotification){
                notificationManagerCompat.notify(1, notification);
            }else {
                notificationManagerCompat.cancel(1);
            }

            return notificationManagerCompat;
        } catch (Exception e) {
            Log.e("Adroll", e.getMessage());
        }

        return null;
    }

    public void openFile(Context context, File f){
        try {
            if (context!=null){
                Intent intent = new Intent();
                intent.setAction(Intent.ACTION_VIEW);
                if (Build.VERSION.SDK_INT < 24) {
                    /* 调用getMIMEType()来取得MimeType */
                    String type = "application/vnd.android.package-archive";
                    /* 设置intent的file与MimeType */
                    intent.putExtra(Intent.EXTRA_NOT_UNKNOWN_SOURCE, true);
                    intent.putExtra(Intent.EXTRA_RETURN_RESULT, true);
                    intent.setDataAndType(Uri.fromFile(f), type);
                } else {
                    Uri uri = FileProvider.getUriForFile(context, "com.mediamain.sdk_flutter_example" + ".basefileprovider", f);
                    intent.setDataAndType(uri, "application/vnd.android.package-archive");
                    intent.putExtra(Intent.EXTRA_NOT_UNKNOWN_SOURCE, true);
                    intent.putExtra(Intent.EXTRA_RETURN_RESULT, true);
                    intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
                }
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                context.startActivity(intent);
            }
        }catch (Exception e){

        }
    }

    public boolean isAppInstall(Context context, String packageName) {
        if (packageName == null || "".equals(packageName)){
            return false;
        }

        try {
            ApplicationInfo info = context.getPackageManager().getApplicationInfo(packageName,
                    PackageManager.GET_UNINSTALLED_PACKAGES);
            return true;
        } catch (PackageManager.NameNotFoundException e) {
            return false;
        }
    }

    public void registerBroadCast(String path, String packageName, String tuiaId){
        if (downloadBroadCast == null){
            downloadBroadCast = new DownloadBroadCast();
            IntentFilter downloadFilter = new IntentFilter();
            downloadFilter.addAction(Intent.ACTION_PACKAGE_ADDED);
            downloadFilter.addDataScheme("package");
            CommonUtils.Companion.getApplication().registerReceiver(downloadBroadCast, downloadFilter);
        }

        downloadBroadCast.setBean(path, packageName, "", tuiaId);
    }

    public void openApp(String packageName){
        try {
            Intent intent = CommonUtils.Companion.getApplication().getPackageManager().getLaunchIntentForPackage(packageName);
            CommonUtils.Companion.getApplication().startActivity(intent);
        }catch (Exception e){

        }
    }
}
