package com.mediamain.sdk_flutter.util

import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import com.mediamain.sdk_flutter.util.CommonUtils.Companion.isEmpty

object GsonUtil {
    private var gson: Gson? = null

    /**
     * 转成json
     *
     * @param object
     * @return
     */
    fun GsonString(`object`: Any?): String? {
        var gsonString: String? = null
        try {
            if (gson != null) {
                gsonString = gson?.toJson(`object`)
            }
        } catch (ignored: Exception) {
        }
        return gsonString
    }

    /**
     * 转成bean
     *
     * @param gsonString
     * @param cls
     * @return
     */
    fun <T> GsonToBean(gsonString: String?, cls: Class<T>?): T? {
        var t: T? = null
        if (isEmpty(gsonString!!)) {
            return null
        }
        try {
            if (gson != null) {
                t = gson?.fromJson(gsonString, cls)
            }
        } catch (e: Exception) {
            return null
        }
        return t
    }

    /**
     * 转成list
     *
     * @param gsonString
     * @param cls
     * @return
     */
    fun <T> GsonToList(gsonString: String?, cls: Class<T>?): List<T>? {
        var list: List<T>? = null
        if (isEmpty(gsonString!!)) {
            return null
        }
        try {
            if (gson != null) {
                list = gson?.fromJson(gsonString, object : TypeToken<List<T>?>() {}.type)
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return list
    }

    /**
     * 转成list中有map的
     *
     * @param gsonString
     * @return
     */
    @Deprecated("")
    fun <T> GsonToListMaps(gsonString: String?): List<Map<String, T>>? {
        var list: List<Map<String, T>>? = null
        try {
            if (gson != null) {
                list = gson?.fromJson(gsonString,
                        object : TypeToken<List<Map<String?, T>?>?>() {}.type)
            }
        } catch (ignored: Exception) {
        }
        return list
    }

    /**
     * 转成map的
     *
     * @param gsonString
     * @return
     */
    fun <T> GsonToMaps(gsonString: String?): Map<String, T>? {
        var map: Map<String, T>? = null
        if (isEmpty(gsonString!!)) {
            return null
        }
        try {
            if (gson != null) {
                map = gson?.fromJson(gsonString, object : TypeToken<Map<String?, T>?>() {}.type)
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return map
    }

    /**
     * 对gosn转化的map特殊处理,主要处理int类型
     *
     * @param maps
     */
    fun HandleIntMaps(maps: MutableMap<String?, Any?>?): Map<String?, Any?>? {
        if (maps != null) {
            for (key in maps.keys) {
                val value = maps[key]
                //判断是否是 .0 结尾
                val isInt = value.toString().substring(value.toString().indexOf(".") + 1, value.toString().length) == "0"
                if (value is Double && isInt) {
                    maps[key] = value.toInt()
                } else {
                    maps[key] = value.toString()
                }
            }
        }
        return maps
    }

    /**
     * 对gosn转化的map特殊处理,主要处理int类型
     *
     * @param maps
     */
    fun HandleSignIntMaps(maps: MutableMap<String?, Any?>?): Map<String?, Any?>? {
        if (maps != null) {
            for (key in maps.keys) {
                val value = maps[key]
                //判断是否是 .0 结尾
                val isInt = value.toString().substring(value.toString().indexOf(".") + 1, value.toString().length) == "0"
                if (value is Double && isInt) {
                    maps[key] = value.toInt()
                } else {
                    maps[key] = value
                }
            }
        }
        return maps
    }

    /**
     * Map转成json
     *
     * @param o
     */
    fun MapToGson(o: Any?): String {
        return if (gson != null) {
            gson!!.toJson(o)
        } else ""
    }

    init {
        if (gson == null) {
            gson = Gson()
        }
    }
}