package com.mediamain.sdk_flutter.util

import android.Manifest
import android.app.Activity
import android.content.Context
import android.content.Context.WIFI_SERVICE
import android.hardware.Sensor
import android.hardware.SensorManager
import android.location.Location
import android.net.wifi.ScanResult
import android.net.wifi.WifiInfo
import android.net.wifi.WifiManager
import android.os.Build
import android.os.Environment
import android.util.Base64
import com.mediamain.sdk_flutter.ImeiPlugin
import com.mediamain.sdk_flutter.model.*
import com.mediamain.sdk_flutter.util.NetworkTypeUtils.NetworkType
import com.mediamain.sdk_flutter.util.NetworkTypeUtils.getNetworkType
import com.mediamain.sdk_flutter.util.cons.CommonConstants
import com.mediamain.sdk_flutter.util.cons.DeviceTypeEnum
import java.io.File


object ParamsUtils {

    /**
     * 基础MD
     */
    fun getMD(activity: Activity, sdkVersion: String?): String? {

        val foxBaseMdBean = MDBean()
        foxBaseMdBean.dataType = "1"
        foxBaseMdBean.os = "Android"
        foxBaseMdBean.api_version = "1.0.0"
        foxBaseMdBean.sdk_version = sdkVersion.toString()
        try {
            foxBaseMdBean.imei = ImeiPlugin.getImei(activity)
            foxBaseMdBean.device_id = ImeiPlugin.getImei(activity)
            foxBaseMdBean.nt = getNetType(activity)

            foxBaseMdBean.oaid = SPUtils.instance?.getString(CommonConstants.KEY_TUIA_SDK_APP_OAID, "")
            //AB分流
//            foxBaseMdBean.setTest_type(FoxBaseTestUtils.getABTest(slotId))
            foxBaseMdBean.packageName = activity.application?.packageName
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return Base64.encodeToString(JSDKGZipUtil.compress(GsonUtil.GsonString(foxBaseMdBean), JSDKGZipUtil.GZIP_ENCODE_UTF_8), Base64.NO_WRAP)
    }

    /**
     * Amass_MD
     */
    fun getAmass(activity: Activity, unCollect: String, appKey: String?, sdkVersion: String?): String? {

        val amassBean = AmassBean()
        amassBean.dataType = "1"
        amassBean.os_type = "Android"
        amassBean.api_version = "1.0.0"
        amassBean.sdk_version = sdkVersion
        amassBean.develop_type = "flutter"


        //time
        if (!unCollect.contains("time")) {
            try {
                amassBean.time = System.currentTimeMillis().toString()
            } catch (e: Exception) {
                amassBean.time = ""
            }
        }
        //app_key
        if (!unCollect.contains("app_key")) {
            amassBean.app_key = appKey
        }
        //package_name
        if (!unCollect.contains("package_name")) {
            amassBean.package_name = getPackageName(activity)
        }
        //imei
        if (!unCollect.contains("imei")) {
            amassBean.imei = getImei(activity)
        }
        //oaid
        if (!unCollect.contains("oaid")) {
            amassBean.oaid = getOaid(activity)
        }
        //tcid
        if (!unCollect.contains("tcid")) {
            amassBean.tcid = getTcid()
        }
        //device_id
        if (!unCollect.contains("device_id")) {
            amassBean.device_id = getImei(activity)
        }
        //gps_longitude
        if (!unCollect.contains("gps_longitude")) {
            amassBean.gps_longitude = getGPSLongitude(activity)
        }
        //gps_latitude
        if (!unCollect.contains("gps_latitude")) {
            amassBean.gps_latitude = getGPSLatitude(activity)
        }
        //net_type
        if (!unCollect.contains("net_type")) {
            amassBean.net_type = getNetType(activity)
        }
        //app_list
        if (!unCollect.contains("app_list")) {
            amassBean.app_list = getAppList(activity)
        }
        //os_version
        if (!unCollect.contains("os_version")) {
            amassBean.os_version = getOSVersion()
        }
        //phone_brand
        if (!unCollect.contains("phone_brand")) {
            amassBean.phone_brand = getPhoneBrand()
        }
        //phone_model
        if (!unCollect.contains("phone_model")) {
            amassBean.phone_model = getPhoneModel()
        }
        //phone_electric
        if (!unCollect.contains("phone_electric")) {
            amassBean.phone_electric = getPhoneElectric(activity)
        }
        //phone_ip
        if (!unCollect.contains("phone_ip")) {
            amassBean.phone_ip = getPhoneIp()
        }
        //net_carrier
        if (!unCollect.contains("net_carrier")) {
            amassBean.net_carrier = getNetCarrier(activity)
        }
        //sensor_info
        if (!unCollect.contains("sensor_info")) {
            amassBean.sensor_info = getSensorInfo(activity)
        }
        //wifi_name
        if (!unCollect.contains("wifi_name")) {
            amassBean.wifi_name = getWifiInfo(activity)
        }
        //wifi_list
        if (!unCollect.contains("wifi_list")) {
            amassBean.wifi_list = getWifiList(activity)
        }
        //is_emulator
        if (!unCollect.contains("is_emulator")) {
            var is_emulator = ""
            try {
                var is_emulator = if (JCommonUtils.isEmulator(activity)) "1," else "0,"
                is_emulator += if (JCommonUtils.isXposedExists()) "2" else ""
            } catch (e: Exception) {

            }
            amassBean.is_emulator = is_emulator
        }

        return Base64.encodeToString(JSDKGZipUtil.compress(GsonUtil.GsonString(amassBean), JSDKGZipUtil.GZIP_ENCODE_UTF_8), Base64.NO_WRAP)
    }

    /**
     * 获取OAID
     */
    fun getOaid(context: Context): String? {
        return SDKOaidUtils.getOAID(context)
    }

    //IMEI
    fun getImei(activity: Activity): String? {
        return ImeiPlugin.getImei(activity)
    }

    //时间戳
    fun getTimestamp(): String? {
        return System.currentTimeMillis().toString()
    }

    //随机数
    fun getNonce(): String? {
        return ((Math.random() * 9 + 1) * 100000).toInt().toString()
    }

    //参数签名
    fun getSignature(activity: Activity, appSecret: String?, nonce: String?, timestamp: String?, sdkVersion: String?): String? {
        val signature: String? = JCommonUtils.sha1("appSecret=" + appSecret + "&md=" + getMD(activity, sdkVersion) + "&nonce=" + nonce + "&timestamp=" + timestamp)
        LogUtils.e("===getSignature===" + signature)
        return signature
    }

    //TCID
    fun getTcid(): String? {
        var tcid: String? = ""
        try {
            tcid = getTcidFromLocal()
            if (tcid?.isEmpty()!!) {
                tcid = SPUtils.instance?.getString(TcIdDataUtils.TCID, "")
                if (tcid?.isEmpty()!!) {
                    tcid = TcIdDataUtils.getUUIDAndMd5(DeviceTypeEnum.IMEI.desc)
                    SPUtils.instance?.setString(TcIdDataUtils.TCID, tcid)
                }
                writeTcidToLocal(tcid)
            } else {
                // 本地读取到，覆盖SP中的tcid值，保证一致
                SPUtils.instance?.setString(TcIdDataUtils.TCID, tcid)
            }
        } catch (e: Exception) {
        }
        return tcid
    }

    fun getCrashData(activity: Activity, dataFrom: Int, mConfigData: String, appkey: String?,
                     crashLog: String?, dataType: String?, isError: Boolean, sdkVersion: String?, crashType: String?): String? {
        val foxBaseCrashBean = CrashBean()
        foxBaseCrashBean.os_type = "Android"
        foxBaseCrashBean.data_type = dataType
        foxBaseCrashBean.data1 = mConfigData
        foxBaseCrashBean.data_from = dataFrom.toString()
        foxBaseCrashBean.time = System.currentTimeMillis().toString()
        foxBaseCrashBean.app_key = appkey
        try {
            foxBaseCrashBean.os_version = Build.VERSION.RELEASE
            foxBaseCrashBean.phone_model = getPhoneModel()
            foxBaseCrashBean.phone_brand = Build.MANUFACTURER
            foxBaseCrashBean.package_name = getPackageName(activity)
            foxBaseCrashBean.sdk_version = sdkVersion
            foxBaseCrashBean.imei = getImei(activity)
            foxBaseCrashBean.device_id = getImei(activity)
            foxBaseCrashBean.net_type = getNetType(activity)
            foxBaseCrashBean.crash_logs = crashLog
            foxBaseCrashBean.oaid = getOaid(activity)
        } catch (e: Exception) {
            e.printStackTrace()
        }
        if (dataType == "1") {
//            val onTime: String = FoxBaseSPUtils.getInstance().getString(FoxTimeRecordTask.LAST_ON_TIME, "")
//            val offTime: String = FoxBaseSPUtils.getInstance().getString(FoxTimeRecordTask.LAST_OFF_TIME, "")
//            foxBaseCrashBean.setLast_on_time(onTime)
//            foxBaseCrashBean.setLast_off_time(offTime)
        }

        //是否授权
        var is_empower = false
        try {
            is_empower = PermissionUtils.isGranted(Manifest.permission.READ_PHONE_STATE)
        } catch (e: Exception) {
            e.printStackTrace()
        }
        foxBaseCrashBean.is_empower = (if (is_empower) "1" else "0")
        foxBaseCrashBean.tcid = getTcid()
        foxBaseCrashBean.crashType = crashType
//        foxBaseCrashBean.setPatchVersion(FoxBaseSPUtils.getInstance().getString(LOCAL_PATCH_VERSION,""));
//        foxBaseCrashBean.setPatchCode(FoxBaseSPUtils.getInstance().getLong(LOCAL_PATCH_CODE,0l) + "");

//        if (isError) {
//            foxBaseCrashBean.setCrashType("3")
//        } else if (dataType == "2") {
//            foxBaseCrashBean.setCrashType(FoxBaseCacheUtils.readString(FoxBaseConstants.KEY_TUIA_SDK_CRASH_INFO_TYPE))
//        }
        return Base64.encodeToString(JSDKGZipUtil.compress(GsonUtil.GsonString(foxBaseCrashBean), JSDKGZipUtil.GZIP_ENCODE_UTF_8), Base64.NO_WRAP)
    }


    /*---------------------------------------------------------Chack_Data-----------------------------------------------------------------------*/
    //包名
    fun getPackageName(context: Context): String? {
        return context.packageName
    }

    //App名称
    fun getAppName(context: Context): String? {
        return CommonUtils.getAppName(getPackageName(context), context);
    }

    //经度 gps_longitude
    fun getGPSLongitude(context: Context): String? {
        var longitude: String = ""
        try {
            val gpsInfo: Location? = CommonUtils.getGPSInfo(context)
            longitude = gpsInfo?.longitude.toString()
        } catch (e: Exception) {
        }
        return longitude
    }

    //纬度 gps_latitude
    fun getGPSLatitude(context: Context): String? {
        var latitude: String = ""
        try {
            val gpsInfo: Location? = CommonUtils.getGPSInfo(context)
            latitude = gpsInfo?.latitude.toString()
        } catch (e: Exception) {
        }
        return latitude
    }

    //操作系统标识 os_type
    fun getOSType(): String? {
        return "Android"
    }

    //网络类型 net_type
    fun getNetType(context: Context): String? {
        var netType: String = ""
        try {
            netType = when (getNetworkType(context)) {
                NetworkType.NETWORK_2G ->
                    "2G"
                NetworkType.NETWORK_3G ->
                    "3G"
                NetworkType.NETWORK_4G ->
                    "4G"
                NetworkType.NETWORK_WIFI ->
                    "wifi"
                NetworkType.NETWORK_5G ->
                    "5G"
                else ->
                    ""
            }
        } catch (e: Exception) {
        }
        return netType
    }

    //app列表 app_list
    fun getAppList(context: Context): String? {
        var appList: String? = ""
        try {
            appList = CommonUtils.getAppList(context)
        } catch (e: Exception) {
        }
        return appList
    }

    //app活跃列表 app_active_list
    fun getActiveList(context: Context): String? {
        val activeList: String? = ""
        return activeList
    }

    //操作系统版本 os_version
    fun getOSVersion(): String? {
        return Build.VERSION.RELEASE
    }

    //手机品牌 phone_brand
    fun getPhoneBrand(): String? {
        return Build.MANUFACTURER
    }

    //手机型号 phone_model
    fun getPhoneModel(): String? {
        var phoneModel = Build.MODEL
        phoneModel = phoneModel?.trim { it <= ' ' }?.replace("\\s*".toRegex(), "") ?: ""
        return phoneModel
    }

    // 手机电量	phone_electric
    fun getPhoneElectric(context: Context): String? {
        return CommonUtils.getBatteryInfo(context).toString() + "%"
    }

    // 手机IPV4地址	phone_ip
    fun getPhoneIp(): String? {
        val phoneIp: String? = CommonUtils.getIPAddress(true)
        return phoneIp
    }

    // 网络运营商	net_carrier
    fun getNetCarrier(context: Context): String? {
        return CommonUtils.getSimOperatorByMnc(context)
    }

    // develop_type	开发方式	是	是	是
    // 分为原生和flutter，
    // 原生：pro；
    // flutter：flutter
    fun getDevelopType(): String? {
        return "flutter"
    }

    /**
     * ChackData
     *
     * 签名数据
     */
    fun getSign(map: Map<String, Any>?, hostKey: String?): String? {
        LogUtils.e("GsonUtil.GsonString(map) === " + GsonUtil.GsonString(map))
//        val result:String = RSACrypt.encryptByPublicKey("随便随便",appSecret)
//        LogUtils.e("加密结果 === " +result)
        return CommonUtils.encryptRSA(GsonUtil.GsonString(map), hostKey)
//        return result//JCommonUtils.encrypt1(GsonUtil.GsonString(map), appSecret)
    }

    /*----------------------------------------------------Private--------------------------------------------------------------------*/

    private fun getSensorInfo(context: Context): List<SensorBean>? {
        //采集传感器信息
        val sensorList: ArrayList<SensorBean> = ArrayList()
        val sm: SensorManager = context.getSystemService(Context.SENSOR_SERVICE) as SensorManager
        val allSensors: List<Sensor> = sm.getSensorList(Sensor.TYPE_ALL)
        if (!CommonUtils.isEmptyList(allSensors)) {
            for (sensor in allSensors) {
                if (sensor.type == Sensor.TYPE_GYROSCOPE || sensor.type == Sensor.TYPE_MOTION_DETECT) {
                    // v3.1.2.0:只收集陀螺仪或运动传感器类型的传感器信息
                    sensorList.add(SensorBean(sensor.name, sensor.type, sensor.vendor,
                            sensor.version, sensor.resolution, sensor.maximumRange, sensor.power))
                }
            }
        }
        return sensorList
    }

    /**
     * 获取Wifi信息
     */
    private fun getWifiInfo(context: Context): String? {
        var wifiId = "-1"
        try {
            wifiId = if (getNetworkType(context) === NetworkType.NETWORK_WIFI) {
                val wifiMgr: WifiManager = context.getSystemService(WIFI_SERVICE) as WifiManager
                val info: WifiInfo = wifiMgr.getConnectionInfo()
                if (info != null) info.getSSID() else "-1"
            } else {
                "1"
            }
        } catch (e: java.lang.Exception) {
            e.printStackTrace()
        }
        return wifiId
    }

    private fun getWifiList(context: Context): List<WifiInfoBean>? {
        val wifiList: MutableList<WifiInfoBean> = ArrayList()
        try {
            val wifiManager = context.getSystemService(WIFI_SERVICE) as WifiManager
            val scanWifiList: List<ScanResult>? = wifiManager.scanResults
            if (scanWifiList != null && scanWifiList.isNotEmpty()) {
                val signalStrength = HashMap<String, Int?>()
                for (i in scanWifiList.indices) {
                    val scanResult: ScanResult = scanWifiList[i]
                    if (scanResult.SSID.isNotEmpty()) {
                        val key: String = scanResult.SSID.toString() + "---" + scanResult.capabilities
                        if (!signalStrength.containsKey(key)) {
                            signalStrength[key] = i
                            wifiList.add(WifiInfoBean(scanResult.SSID, scanResult.BSSID, scanResult.capabilities, scanResult.level))
                        }
                    }
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return wifiList
    }

    private fun writeTcidToLocal(tcid: String?) {
        if (PermissionUtils.isGranted(Manifest.permission.WRITE_EXTERNAL_STORAGE)) {
            val directory = getSdcardPath().toString() + File.separator + CommonConstants.DIR_CONFIG
            JFileUtils.createOrExistsDir(directory)
            val configFile = File(directory, "config")
            JFileUtils.createOrExistsFile(configFile)
            val content = "tcid=$tcid"
            JFileIOUtils.writeFileFromString(configFile, content, false)
        }
    }

    private fun getTcidFromLocal(): String? {
        var tcid = ""
        // check permission
        tcid = if (PermissionUtils.isGranted(Manifest.permission.READ_EXTERNAL_STORAGE)) {
            val directory: String = getSdcardPath() + File.separator + CommonConstants.DIR_CONFIG
            JFileUtils.createOrExistsDir(directory)
            val configFile = File(directory, "config")
            JFileUtils.createOrExistsFile(configFile)
            val configList: List<String> = JFileIOUtils.readFile2List(configFile, 0, 0x7FFFFFFF, null)
            if (configList == null || configList.isEmpty()) {
                ""
            } else {
                configList[0].substring("tcid=".length)
            }
        } else {
            ""
        }
        return tcid
    }

    fun getSdcardPath(): String? {
        return if (Environment.MEDIA_MOUNTED.equals(Environment.getExternalStorageState())) {
            Environment.getExternalStorageDirectory().getAbsolutePath()
        } else "/storage/emulated/0"
    }

    fun getSha1(input: String): String? {

        return JCommonUtils.sha1(input)
    }
}