package com.mediamain.sdk_flutter.util

import android.content.Context
import android.text.TextUtils
import com.mediamain.sdk_flutter.util.CommonUtils.Companion.isEmpty
import com.mediamain.sdk_flutter.util.SPUtils.Companion.instance
import com.mediamain.sdk_flutter.util.cons.CommonConstants
import java.lang.reflect.InvocationHandler
import java.lang.reflect.Method
import java.lang.reflect.Proxy
import java.util.concurrent.CountDownLatch
import java.util.concurrent.TimeUnit

object SDKOaidUtils {
    private var IIdentifierListener = "com.bun.miitmdid.core.IIdentifierListener"
    private var IdSupplier = "com.bun.miitmdid.supplier.IdSupplier"
    private const val TAG = "FoxBaseOAUtils"
    private var oaid: String? = ""
    private var countDownLatch: CountDownLatch? = null

    /**
     * 获取 OAID 接口
     * 该接口是同步接口，可能会导致线程阻塞，建议在子线程中使用
     *
     * @param context Context
     * @return OAID
     */
    fun getOAID(context: Context): String? {
        try {
            countDownLatch = CountDownLatch(1)
            if (TextUtils.isEmpty(oaid)) {
                oaid = instance!!.getString(CommonConstants.KEY_TUIA_SDK_APP_OAID, "")
                if (TextUtils.isEmpty(oaid)) {
                    getOAIDReflect(context, 3)
                } else {
                    return oaid
                }
            } else {
                return oaid
            }
            try {
                countDownLatch!!.await(1500, TimeUnit.MILLISECONDS)
            } catch (ignored: Exception) {
            }
            return oaid
        } catch (ex: Exception) {
        }
        return ""
    }

    /**
     * 通过反射获取 OAID
     * 结果返回的 ErrorCode 如下：
     * 1008611：不支持的设备厂商
     * 1008612：不支持的设备
     * 1008613：加载配置文件出错
     * 1008614：获取接口是异步的，结果会在回调中返回，回调执行的回调可能在工作线程
     * 1008615：反射调用出错
     * @param context Context
     * @param retryCount 重试次数
     */
    private fun getOAIDReflect(context: Context, retryCount: Int) {
        var retryCount = retryCount
        try {
            if (retryCount <= 0) {
                return
            }
            val INIT_ERROR_RESULT_DELAY = 1008614
            // 初始化 Library
            if (IdSupplier != "com.bun.miitmdid.interfaces.IdSupplier") { //兼容1.0.22版本
                val jLibrary = Class.forName("com.bun.miitmdid.core.JLibrary")
                val initEntry = jLibrary.getDeclaredMethod("InitEntry", Context::class.java)
                initEntry.invoke(null, context)
            }
            val identifyListener = Class.forName(IIdentifierListener)
            // 创建 OAID 获取实例
            val handler = IdentifyListenerHandler()
            val iIdentifierListener = Proxy.newProxyInstance(context.classLoader, arrayOf(identifyListener), handler)
            // 初始化 SDK
            val midSDKHelper = Class.forName("com.bun.miitmdid.core.MdidSdkHelper")
            val initSDK = midSDKHelper.getDeclaredMethod("InitSdk", Context::class.java, Boolean::class.javaPrimitiveType, identifyListener)
            val errCode = initSDK.invoke(null, context, true, iIdentifierListener) as Int
            if (errCode != INIT_ERROR_RESULT_DELAY) {
                getOAIDReflect(context, --retryCount)
                if (retryCount == 0) {
                    countDownLatch!!.countDown()
                }
            }

            /*
             * 此处是为了适配三星部分手机，根据 MSA 工作人员反馈，对于三星部分机型的支持有 bug，导致
             * 返回 1008614 错误码，但是不会触发回调。所以此处的逻辑是，两秒之后主动放弃阻塞。
             */
            //TODO 线程池待处理
//            FoxBaseThreadUtils.getCachedPool().execute(new Runnable() {
//                @Override
//                public void run() {
//                    try {
//                        Thread.sleep(1500);
//                    } catch (InterruptedException e) {
//                    }
//                    countDownLatch.countDown();
//                }
//            });
            object : Thread() {
                override fun run() {
                    super.run()
                    try {
                        sleep(1500)
                    } catch (e: InterruptedException) {
                    }
                    countDownLatch!!.countDown()
                }
            }.start()
        } catch (ex: Exception) {
            --retryCount
            //解决1.0.12和1.0.13的包名重构问题com.bun.miitmdid.interfaces.IIdentifierListener
            IIdentifierListener = "com.bun.supplier.IIdentifierListener"
            IdSupplier = "com.bun.supplier.IdSupplier"
            if (retryCount == 1) { //兼容1.0.22版本
                IIdentifierListener = "com.bun.miitmdid.interfaces.IIdentifierListener"
                IdSupplier = "com.bun.miitmdid.interfaces.IdSupplier"
            }
            getOAIDReflect(context, retryCount)
            if (retryCount == 0) { // 对于没有集成 jar 包，尝试后为 0
                countDownLatch!!.countDown()
            }
        }
    }

    internal class IdentifyListenerHandler : InvocationHandler {
        @Throws(Throwable::class)
        override fun invoke(proxy: Any, method: Method, args: Array<Any>): Any? {
            try {
                if ("OnSupport" == method.name) {
                    if (args[0] as Boolean) {
                        val idSupplier = Class.forName(IdSupplier)
                        val getOAID = idSupplier.getDeclaredMethod("getOAID")
                        oaid = getOAID.invoke(args[1]) as String
                        //Log.d(TAG, "oaid:" + oaid);
                        LogUtils.e("===oaid===" +oaid);
                                if (!isEmpty(oaid!!)) {
                            instance!!.setString(CommonConstants.KEY_TUIA_SDK_APP_OAID, oaid)
                        }
                    }
                    countDownLatch!!.countDown()
                }
            } catch (ex: Exception) {
                countDownLatch!!.countDown()
            }
            return null
        }
    }
}