
import 'dart:async';

import 'package:flutter/material.dart';
import 'package:sdk_flutter/plugin/DownloadPlugin.dart';

class DownloadPage extends StatefulWidget{

  String _url = "https://wppkg.baidupcs.com/issue/netdisk/apk/BaiduNetdisk_11.9.7.apk";
  String _packageName = "com.baidu.netdisk";
  String _tuiaId = "1234";

  @override
  State<StatefulWidget> createState() {
    return _DownloadPageState();
  }
}

class _DownloadPageState extends State<DownloadPage> with SingleTickerProviderStateMixin {

  AnimationController _controller;
  Timer _timer;

  double progress = 0.0;

  @override
  void initState() {
    super.initState();
    _controller = new AnimationController(vsync: this, duration: Duration(seconds: 5));
    _controller.addListener(() => {setState(() => {})});
    onAnimationStart();

    _timer = Timer.periodic(Duration(milliseconds: 100), (timer) {
      progress = DownloadPlugin.getProgress(widget._url, widget._packageName, widget._tuiaId);
      setState(() {

      });
    });
  }

  void onAnimationStart(){
    _controller.forward(from: 0.0);
  }

  @override
  void dispose() {
    super.dispose();
    _controller.dispose();
    _timer.cancel();
  }

  @override
  Widget build(BuildContext context) {
    final size = MediaQuery.of(context).size;
    final width = size.width;
    final height = size.height;

    return SingleChildScrollView(
      child: Column(
        children: <Widget>[
          SizedBox(
            width: width,
            height: 100,
          ),

          //线性进度条进度动画
          Padding(
              padding: EdgeInsets.all(16),
              child: LinearProgressIndicator(
                backgroundColor: Colors.grey[200],
                valueColor: ColorTween(begin: Colors.grey, end: Colors.red).animate(_controller),
                value: progress / 100,
              ),
          ),

          Column(
            mainAxisAlignment: MainAxisAlignment.center,
            children: <Widget>[
              MaterialButton(
                child: Text("普通下载"),
                padding: EdgeInsets.all(10),
                onPressed: (){
                  DownloadPlugin.defaultDownload(widget._url, widget._packageName, widget._tuiaId);
                },
                disabledColor: Colors.blueGrey,
                color: Colors.pink,
                textColor: Colors.white,
                height: 40.0,
                minWidth: 100.0,
              ),
              MaterialButton(
                child: Text("静默下载"),
                onPressed: (){
                  DownloadPlugin.backgroundDownload(widget._url, widget._packageName, widget._tuiaId);
                },
                padding: EdgeInsets.all(10),
                disabledColor: Colors.blueGrey,
                color: Colors.pink,
                textColor: Colors.white,
                height: 40.0,
                minWidth: 100.0,
              ),
              MaterialButton(
                child: Text("显示通知栏"),
                onPressed: (){
                  DownloadPlugin.showNotification(true);
                },
                padding: EdgeInsets.all(10),
                disabledColor: Colors.blueGrey,
                color: Colors.pink,
                textColor: Colors.white,
                height: 40.0,
                minWidth: 100.0,
              ),
              MaterialButton(
                child: Text("安装"),
                onPressed: (){
                  DownloadPlugin.installApp(widget._url, widget._packageName, widget._tuiaId);
                },
                padding: EdgeInsets.all(10),
                disabledColor: Colors.blueGrey,
                color: Colors.pink,
                textColor: Colors.white,
                height: 40.0,
                minWidth: 100.0,
              ),
              MaterialButton(
                child: Text("暂停下载"),
                onPressed: (){
                  // DownloadPlugin.cancelTask();
                },
                padding: EdgeInsets.all(10),
                disabledColor: Colors.blueGrey,
                color: Colors.pink,
                textColor: Colors.white,
                height: 40.0,
                minWidth: 100.0,
              ),
              MaterialButton(
                child: Text("继续下载"),
                onPressed: (){
                  // DownloadPlugin.continueTask();
                },
                padding: EdgeInsets.all(10),
                disabledColor: Colors.blueGrey,
                color: Colors.pink,
                textColor: Colors.white,
                height: 40.0,
                minWidth: 100.0,
              ),
            ],
          ),
        ],
      ),
    );
  }
}