//
//  TATClient.h
//  TATMediaSDK
//
//  Created by wuleslie on 2019/12/4.
//  Copyright © 2019 wuleslie. All rights reserved.
//

#import <Foundation/Foundation.h>

@interface TATClient : NSObject
/**
 * 应用的Key
 */
@property(nonatomic, readonly) NSString *appKey;
/**
 * 应用的secret
 */
@property(nonatomic, readonly) NSString *appSecret;
/**
 * 媒体是否有初始化SDK
 */
@property(nonatomic, readonly, assign) BOOL isInitialized;
/**
 * 媒体应用ID
 */
@property(nonatomic, copy) NSString *appId;
/**
 * 广告专用用户标识符，MD5加密的
 */
@property(nonatomic, readonly) NSString *idfa;
/**
 * 广告专用用户标识符，明文
 */
@property(nonatomic, readonly) NSString *idfa_original;
/**
 * 包名bundle identifier
 */
@property(nonatomic, readonly) NSString *bundleIdentifier;
/**
 * 系统名称，固定返回iOS
 */
@property(nonatomic, readonly) NSString *systemName;
/**
 * 系统版本号
 */
@property(nonatomic, readonly) NSString *systemVersion;
/**
 * 应用版本,返回CFBundleVersion值
 */
@property(nonatomic, readonly) NSString *appVersion;

/**
 * 应用build号
 */
@property(nonatomic, readonly) NSString *appBuild;
/**
 * 当前网络类型 1.有线 2.wifi 3.3G 4.4G 5.5G
 */
@property(nonatomic, readonly) NSString *networkType;

/**
 * 屏幕尺寸
 */
@property(nonatomic, readonly) NSString *screenSize;
/**
 * 屏幕分辨率
 */
@property(nonatomic, readonly) NSString *screenResolution;
/**
 * 屏幕ppi
 */
@property(nonatomic, readonly) NSString *scale;
/**
 * 安装路径
 */
@property(nonatomic, readonly) NSString *appPath;
/**
 * 应用名称
 */
@property(nonatomic, readonly) NSString *appName;
/**
 * 设备ID
 */
@property(nonatomic, readonly) NSString *deviceId;
/**
 * 地理位置
 */
@property(nonatomic, readonly) NSString *location;
/**
 * 本机外网IP
 */
@property(nonatomic, readonly) NSString *deviceIP;

/**
 设备分辨率
 */
@property (nonatomic, readonly) NSString *deviceResolution;

/**
 纬度
 */
@property (nonatomic, copy) NSString *latitude;

/**
 经度
 */
@property (nonatomic, copy) NSString *longitude;

/**
 所用SDK版本号
 */
@property (nonatomic, copy) NSString *sdkVersion;

/**
 SDK版本号数字
 */
@property (nonatomic, assign) NSInteger sdkVersionCode;
/**
 活动页面是否支持关闭功能，如果支持，则需要在活动页加上关闭按钮
 */
@property (nonatomic, assign) BOOL isSupportClose;
/**
 是否允许收集用户信息
 */
@property (nonatomic, assign) BOOL collectEnable;
/**
 收集用户信息的时间间隔，单位：分钟
 */
@property (nonatomic, assign) NSInteger collectFrequency;
/**
 需要屏蔽的数据key值列表，即这些字段不采集
 */
@property (nonatomic, strong) NSArray *collectShieldFields;
/**
  是否上报关闭App的事件，YES:上报 NO：不上报
 */
@property (nonatomic, assign) BOOL closeEventEnable;
/**
 数据采集来源：1 - 原生插屏，其他-都是大盘SDK，这里就用0表示
 */
@property (nonatomic, copy) NSString *dataFromType;
/**
 禁止采集的数据列表
 */
@property (nonatomic, strong) NSArray *blockList;
/**
 自定义设备唯一标识符
 */
@property (nonatomic, strong) NSString *tcid;
/**
 授权状态
 0：用户未授权
 1：用户已授权
 2：其他
 */
@property (nonatomic, strong) NSString *authStatus;
/**
 * 获取目前的app信息，需要使用registerApp方法先注册app，否则返回nil
 */
+ (TATClient *)current;

+ (instancetype)registerWithSecret:(NSString *)secret appKey:(NSString *)appKey;

- (void)getCurrentLocation;

@end

