//
//  TATLocationManager.m
//  TATMediaSDK
//
//  Created by wuleslie on 2019/12/18.
//  Copyright © 2019 wuleslie. All rights reserved.
//

#import "TATLocationManager.h"

static TATLocationManager *sharedInstance = nil;

@interface TATLocationManager () <CLLocationManagerDelegate>

@property (nonatomic, strong) CLLocationManager *manager;
@property (nonatomic, copy) TATLocationSuccess successBlock;
@property (nonatomic, copy) TATLocationFailure failureBlock;

@end

@implementation TATLocationManager


+ (instancetype)sharedInstance {
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        sharedInstance = [[self alloc] init];
    });
    return sharedInstance;
}

- (instancetype)init {
    self = [super init];
    if (self) {
        _manager = [[CLLocationManager alloc] init];
        _manager.delegate = self;
        _manager.desiredAccuracy = kCLLocationAccuracyBest;
    }
    return self;
}

+ (void)getLocationWithSuccess:(TATLocationSuccess)success failure:(TATLocationFailure)failure {
    if ([CLLocationManager authorizationStatus] != kCLAuthorizationStatusAuthorizedAlways && [CLLocationManager authorizationStatus] != kCLAuthorizationStatusAuthorizedWhenInUse) {
        // 没有授权，则不请求
        !failure ?: failure(nil);
        return;
    }
    [[TATLocationManager sharedInstance] getLocationWithSuccess:success failure:failure];
}

- (void)getLocationWithSuccess:(TATLocationSuccess)success failure:(TATLocationFailure)failure {\
    self.successBlock = success;
    self.failureBlock = failure;
    [self.manager startUpdatingLocation];
}


+ (void)stop {
    if (sharedInstance) {
        [[TATLocationManager sharedInstance].manager stopUpdatingLocation];
    }
}

#pragma mark - CLLocationManagerDelegate

- (void)locationManager:(CLLocationManager *)manager didUpdateLocations:(NSArray<CLLocation *> *)locations {
    CLLocation *location = [locations firstObject];
    CLLocationCoordinate2D coordinate = location.coordinate;
    CLLocationDegrees latitude = coordinate.latitude;
    CLLocationDegrees longitude = coordinate.longitude;
    !self.successBlock ?: self.successBlock(latitude, longitude);
}

- (void)locationManager:(CLLocationManager *)manager
       didFailWithError:(NSError *)error {
    !self.failureBlock ?: self.failureBlock(error);
}

@end
