//
//  TATSdkFlutterPlugin.m
//  sdk_flutter
//
//  Created by xuesong on 2021/5/21.
//

#import "TATSdkFlutterPlugin.h"
#import "TATCocoaSecurity.h"
#import "NSString+TATBase.h"
#import "TATStorageHelper.h"
#import "NSDate+TATFormatter.h"
#import "TATUserDataHelper.h"
#import "TATClient.h"
#import "TATRSAEncryptor.h"
#import "NSObject+TATModel.h"
#import "HAIJIANPPNetworkReachabilityManager.h"

@interface TATSdkFlutterPlugin ()

@end

@implementation TATSdkFlutterPlugin

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar> *)registrar {
    FlutterMethodChannel *channel = [FlutterMethodChannel methodChannelWithName:@"sdk_flutter" binaryMessenger:[registrar messenger]];
    TATSdkFlutterPlugin *instance = [[TATSdkFlutterPlugin alloc] init];
    [registrar addMethodCallDelegate:instance channel:channel];
    
    [[HAIJIANPPNetworkReachabilityManager sharedManager] startMonitoring];
}

- (void)handleMethodCall:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSString *callMethod = call.method;
    if ([callMethod isEqualToString:@"getTcid"]) {
        result([self tcid]);
    } else if ([callMethod isEqualToString:@"getImei"]) {
        result(@"");
    } else if ([callMethod isEqualToString:@"getPhoneBrand"]) {
        result(@"iPhone");
    } else if ([callMethod isEqualToString:@"getPhoneModel"]) {
        result(@"iPhone");
    } else if ([callMethod isEqualToString:@"getOaid"]) {
        result(@"");
    } else if ([callMethod isEqualToString:@"getSdkVersion"]) {
        result(@"1.0.0");
    } else if ([callMethod isEqualToString:@"getOSType"]) {
        result(@"iOS");
    } else if ([callMethod isEqualToString:@"getSha1"]) {
        NSDictionary *args = call.arguments;
        result([self signatureForConfigParams:args[@"sha1Map"]]);
    } else if ([callMethod isEqualToString:@"getIDFA"]) {
        NSString *idfa_md5 = [[TATCocoaSecurity md5:[TATStorageHelper idfa_original]] hexLower];
        result(idfa_md5);
    } else if ([callMethod isEqualToString:@"getDeviceId"]) {
        result([TATStorageHelper idfa_original]);
    } else if ([callMethod isEqualToString:@"getAmassMd"]) {
        NSDictionary *args = call.arguments;
        [TATClient current].sdkVersion = args[@"sdkVersion"];
        NSString *mdString = [self amassMDWithAppKey:args[@"appKey"] blockJson:args[@"unCollect"]];
        result(mdString);
    } else if ([callMethod isEqualToString:@"time"] || [callMethod isEqualToString:@"getTimestamp"]) {
        result([NSString stringWithFormat:@"%@", @([NSDate tat_currentTimestamp])]);
    } else if ([callMethod isEqualToString:@"getPackageName"]) {
        result([[NSBundle mainBundle] infoDictionary][@"CFBundleIdentifier"]);
    } else if ([callMethod isEqualToString:@"getAppName"]) {
        result([[NSBundle mainBundle] infoDictionary][@"CFBundleName"]);
    } else if ([callMethod isEqualToString:@"getNonce"]) {
        int randomNumber = [[self class] randomNumber:100000 to:1000000];
        NSString *nonce = [NSString stringWithFormat:@"%d", randomNumber];
        result(nonce);
    } else if ([callMethod isEqualToString:@"getOSVersion"]) {
        result([[UIDevice currentDevice] systemVersion]);
    } else if ([callMethod isEqualToString:@"getGPSLongitude"]) {
        result([TATClient current].longitude);
    } else if ([callMethod isEqualToString:@"getGPSLatitude"]) {
        result([TATClient current].latitude);
    } else if ([callMethod isEqualToString:@"getNetType"]) {
        result([TATClient current].networkType);
    } else if ([callMethod isEqualToString:@"getSignature"]) {
        result([self signatureWithParams:call.arguments]);
    } else if ([callMethod isEqualToString:@"getSign"]) {
        result([self rsaStringFromParams:call.arguments]);
    } else if ([callMethod isEqualToString:@"getMD"]) {
        [TATClient current].sdkVersion = call.arguments[@"sdkVersion"];
        result([TATUserDataHelper sevingMDString]);
    } else if ([callMethod isEqualToString:@"getCrashData"]) {
        [TATClient current].sdkVersion = call.arguments[@"sdkVersion"];
        [TATClient registerWithSecret:nil appKey:call.arguments[@"appKey"]];
        result([TATUserDataHelper crashMDWithCrashLog:call.arguments[@"crashLog"] dataType:call.arguments[@"dataType"] crashType:call.arguments[@"crashType"]]);
    } else {
        result(@"");
    }
}

- (NSString *)tcid {
    NSString *tcid = [TATStorageHelper tcid];
    if (![tcid tat_isNotEmpty]) {
        NSString *uuid = [TATSdkFlutterPlugin uuidString];
        NSString *mixedId = [NSString stringWithFormat:@"%lld%@", [NSDate tat_currentTimestamp], uuid];
        NSString *md5Id = [[TATCocoaSecurity md5:mixedId] hexLower];
        if (md5Id.length < 10) { // exception
            return @"";
        }
        NSString *prefixCode = @"1020";
        tcid = [md5Id stringByReplacingCharactersInRange:NSMakeRange(0, 4) withString:prefixCode];
        [TATStorageHelper cacheTcid:tcid];
    }
    return tcid;
}

+ (NSString *)uuidString {
    CFUUIDRef uuid_ref = CFUUIDCreate(NULL);
    CFStringRef uuid_string_ref = CFUUIDCreateString(NULL, uuid_ref);
    NSString *uuid = [NSString stringWithString:(__bridge NSString *)uuid_string_ref];
    CFRelease(uuid_ref);
    CFRelease(uuid_string_ref);
    return [uuid lowercaseString];
}

// 获取[from, to)范围的随机数
+ (int)randomNumber:(int)from to:(int)to {
    return (int)(from + (arc4random() % (to - from + 1)));
}

- (NSString *)amassMDWithAppKey:(NSString *)appKey blockJson:(NSString *)blockJson {
    [TATClient registerWithSecret:nil appKey:appKey];
    NSData *jsonData = [blockJson dataUsingEncoding:NSUTF8StringEncoding];
    NSError *err;
    NSArray *blockList = [NSJSONSerialization JSONObjectWithData:jsonData options:NSJSONReadingMutableContainers error:&err];
    [TATUserDataHelper configBlockList:blockList];
    return [TATUserDataHelper amassMdStringWithBlockList:blockList];
}

- (NSString *)signatureWithParams:(NSDictionary *)params {
    NSString *appSecret = params[@"appSecret"];
    NSString *nonce = params[@"nonce"];
    NSString *timestamp = params[@"timestamp"];
    NSString *mdString = [TATUserDataHelper sevingMDString];
    // 组装，采用sha1加密生成签名
    NSString *pinnedString = [NSString stringWithFormat: @"appSecret=%@&md=%@&nonce=%@&timestamp=%@", appSecret, mdString, nonce, timestamp];
    NSString *signature = [TATCocoaSecurity sha1:pinnedString].hexLower;
    return signature;;
}

- (NSString *)rsaStringFromParams:(NSDictionary *)params {
    NSString *json = [params[@"maps"] tat_modelToJSONString];
    NSString *publicKey = params[@"hostKey"];
    return [TATRSAEncryptor encryptString:json publicKey:publicKey];
}

- (NSString *)signatureForConfigParams:(NSDictionary *)params {
    // 按key值排序，生成signature
    NSArray *sortedKeys = [[params allKeys] sortedArrayUsingSelector:@selector(compare:)];
    NSString *pinnedString = @"";
    for (NSString *key in sortedKeys) { // 有序拼接
        pinnedString = [pinnedString stringByAppendingFormat:@"%@=%@&", key, params[key]];
    }
    pinnedString = [pinnedString substringToIndex:pinnedString.length - 1];
    // 参数拼接字符串进行SHA1加密后得到签名signature
    NSString *signature = [TATCocoaSecurity sha1:pinnedString].hexLower;
    return signature;
}

@end
