import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:sdk_flutter/global/cons/SDKType.dart';
import 'package:sdk_flutter/http/api/AdApi.dart';
import 'package:sdk_flutter/http/api/EnterApi.dart';
import 'package:sdk_flutter/model/AdBean.dart';
import 'package:sdk_flutter/model/ResultBean.dart';
import 'package:sdk_flutter/global/AdCallBack.dart';

import 'ActivityWebPage.dart';

class CustomSuperAd {
  String _mSlotId;
  String _mUserId;
  BuildContext _mContext;
  ResultBean<AdBean> _mData;

  // AdClickCallBack _onClickCallBack;
  AdLoadSuccess _onAdLoadSuccess;
  LoadFailedCallBack _onLoadFailedCallBack;
  AdActivityCloseCallBack _onAdActivityCloseCallBack;
  bool _isClicked = false;


  //显示素材（对外函数）
  Future<ResultBean<AdBean>> loadAd({
    @required BuildContext pContext,
    @required String slotId,
    @required String userId,
    AdLoadSuccess onAdLoadSuccess, //广告请求成功
    LoadFailedCallBack onLoadFailedCallBack,
    AdActivityCloseCallBack onAdActivityCloseCallBack,
  }) async {
    _mContext = pContext;
    _mSlotId = slotId;
    _mUserId = userId;
    _onAdLoadSuccess = onAdLoadSuccess;
    _onLoadFailedCallBack = onLoadFailedCallBack;
    _onAdActivityCloseCallBack = onAdActivityCloseCallBack;
    //数据请求
    _mData = await _requestData();

    //数据拉取成功回调
    if(null != _mData && null != _onAdLoadSuccess){
      _onAdLoadSuccess(_mData);
    }

    return _mData;
  }

  openActivity(){
    if(_mContext == null || _mData == null) return;
    _adClick(_mContext, _mData);
  }

  onExposed(){
    _reportData(0);
  }

  onClick(){
    _reportData(1);
  }

  //拿数据
  Future<ResultBean<AdBean>> _requestData() async {
    var adSlot = _mSlotId;
    var userId = _mUserId;

    var params = {
      "deviceSlotExpCount": "1",
      "slotAccessType": "1",
      "slotSceneType": SDKType.CUSTOMER_TM,
    };

    ResultBean<AdBean> result = await AdApi.instance
        .loadAd(adSlot: adSlot, userId: userId, param: params, onLoadFailedCallBack: _onLoadFailedCallBack);
    _isClicked = false;
    return result;
  }

  //按钮点击
  _adClick(BuildContext pContext, ResultBean<AdBean> data) async {
      _openActivity(pContext, data);
  }

  //打开活动
  _openActivity(BuildContext pContext, ResultBean<AdBean> data) async {
    var inContext = pContext;
    if (null == inContext) return;

    // 点击上报
    if (!_isClicked) {
      _reportData(1);
      _isClicked = true;
    }

    var result = await Navigator.push(
        inContext,
        MaterialPageRoute(
          builder: (inContext) => ActivityWebPage(
            data: data,
          ),
        ));

    if (result.toString() == "退出") {
      // createDraggable(offset: _endOffset, pContext: inContext);
      if (_onAdActivityCloseCallBack != null) {
        _onAdActivityCloseCallBack("");
      }
    }
  }

  void _reportData(int reportType) {
    var param = {"slotAccessType": "0", "actualInteractiveType": "0"};
    EnterApi.instance.reportData(reportType, _mData, param);
  }
}
