import 'dart:ui';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:sdk_flutter/http/api/AdApi.dart';
import 'package:sdk_flutter/http/api/EnterApi.dart';
import 'package:sdk_flutter/model/AdBean.dart';
import 'package:sdk_flutter/model/ResultBean.dart';
import 'package:sdk_flutter/plugin/DecorationPlugin.dart';
import 'package:sdk_flutter/util/CommonUtils.dart';
import 'package:sdk_flutter/util/CrashUtils.dart';
import 'package:sdk_flutter/util/ImageUtils.dart';
import 'package:sdk_flutter/util/log_util.dart';
import 'package:sdk_flutter/global/AdCallBack.dart';
import 'package:sdk_flutter/global/cons/SDKType.dart';

import 'ActivityWebPage.dart';
import 'AdIcon.dart';

class IconWidget extends StatefulWidget {
  IconWidget({
    @required this.context,
    @required this.slotId,
    @required this.userId,
    this.width = 60.0, //宽
    this.height = 60.0, //高
    this.offset = const Offset(0.0, 0.0), //初始位置
    this.animation, //动画
    this.decoration, //样式 (暂不支持)
    this.isDraggable = false, //是否可以拖拽
    this.isRounded = false, //是否为圆角
    this.isShow = true, //是否显示
    this.radius = 0.0, //圆角度
    this.boxFit = BoxFit.fill, //图片填充方式
    this.onClickCallBack,
    this.onLoadSuccessCallBack,
    this.onAdCloseCallBack,
    this.onLoadFailedCallBack,
    this.onAdActivityCloseCallBack,
  }) : assert(context != null);

  final BuildContext context;
  final String slotId;
  final String userId;
  final double width;
  final double height;
  final Offset offset;
  final Animation animation; //动画
  final Decoration decoration; //样式 (暂不支持)
  final bool isDraggable; //是否可以拖拽
  final bool isRounded; //是否为圆角
  final bool isShow; //是否显示
  final double radius; //圆角度
  final BoxFit boxFit; //图片填充方式
  final AdClickCallBack onClickCallBack;
  final LoadSuccessCallBack onLoadSuccessCallBack;
  final AdCloseCallBack onAdCloseCallBack;
  final LoadFailedCallBack onLoadFailedCallBack;
  final AdActivityCloseCallBack onAdActivityCloseCallBack;

  _IconWidgetState _mState;

  @override
  _IconWidgetState createState() {
    _mState = _IconWidgetState();
    return _mState;
  }

  _IconWidgetState getState() {
    return _mState;
  }
}

class _IconWidgetState extends State<IconWidget> {
  OverlayEntry _holder;
  ResultBean<AdBean> _mData;
  Widget _showView;
  Offset _endOffset;
  Offset _startOffset;
  bool _isVisibility = true;
  bool _isClicked = false;

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    if (!_isVisibility) {
      _isVisibility = true;
      return Container(
        width: 0.0,
        height: 0.0,
      );
    }
    init();
    CommonUtils.getLastDateTime();
    remove(); //移除悬浮窗
    return FutureBuilder(
      future: _requestData(),
      builder: (context, snapshot) {
        switch (snapshot.connectionState) {
          case ConnectionState.none:
            debugPrint("---------ConnectionState.none----------");
            break;
          case ConnectionState.active:
            debugPrint("---------ConnectionState.active----------");
            break;
          case ConnectionState.waiting:
            debugPrint("---------ConnectionState.waiting----------");
            break;
          case ConnectionState.done:
            debugPrint("---------ConnectionState.done----------");
            if (snapshot.hasData) {
              // 通过数据构建界面
              _mData = snapshot.data;
              _isClicked = false;
              debugPrint("图片: ${_mData?.data?.imageUrl}");
              return getShowWidget(context);
            } else {
              // 显示加载Loading，直到数据返回时停止
              debugPrint('构建加载Loading');
              return Container(); //CircularProgressIndicator();
            }
            break;
        }
        return Container();
      },
    );
  }

  void init() {
    _endOffset = widget.offset;
    _startOffset = widget.offset;
    _isVisibility = true;
  }

  void remove() {
    try {
      _holder?.remove();
      _holder = null;
    } catch (e) {}
  }

  //获取显示的Widget
  Widget getShowWidget(BuildContext context) {
    var iContext = context;
    if (null == iContext) return Container();
    if (null == _mData) return Container();

    //原型
    var circleView = Container(
      width: widget.width,
      height: widget.height,
      decoration: _getDecoration(_mData?.data?.imageUrl),
      child: MaterialButton(
        onPressed: () {
          _adClick(iContext, _mData);
        },
      ),
    );
    var cView = Stack(
      alignment: Alignment.centerRight,
      children: [
        circleView,
        AdIcon.getAdIcon(
            _mData?.data?.isVisibleOfIcon,
            _mData?.data?.isVisibleOfCloseButton,
            widget.height, adCloseCallBack: () {
          this._showView = null;
          // _showView = Container(width: 0.0,height: 0.0,);
          setVisibility(isVisibility: false, pContext: iContext);
          if (null != widget.onAdCloseCallBack) {
            widget.onAdCloseCallBack();
          }
        }),
      ],
    );

    var rView = Visibility(
      child: cView,
      visible: _isVisibility,
    );
    debugPrint("_isVisibility===============${_isVisibility}");
    this._showView = rView; //circleView; //显示Widget

    //创建一个OverlayEntry对象（Positioned）
    OverlayEntry overlayEntry = new OverlayEntry(builder: (inContext) {
      return Positioned(top: _startOffset.dy, child: _buildDraggable(iContext));
    });
    // //往Overlay中插入插入OverlayEntry(悬浮)
    if (widget.isDraggable) {
      Future.delayed(Duration(milliseconds: 500), () {
        Overlay.of(iContext).insert(overlayEntry);
      });
    }
    _holder = overlayEntry;

    return widget.isDraggable ? Container() : _showView;
  }

  //提供默认的Decoration
  Decoration _getDecoration(String imageUrl) {
    if (CommonUtils.isEmpty(imageUrl)) {
      return null;
    }
    Decoration dImage = DecorationPlugin.getCommonDecoration(
        dImage: ImageUtils.getNetworkImage(imageUrl, fit: BoxFit.fill,
            error: (exception, stackTrace) {
          debugPrint(
              "加载错误 exception = ${exception.toString()} stackTrace = ${stackTrace.toString()}");
          setVisibility(isVisibility: false, pContext: context);
        }),
        isCircle: widget.isRounded,
        radius: widget.radius);
    _reportData(0);
    if (widget.onLoadSuccessCallBack != null) {
      widget.onLoadSuccessCallBack();
    }
    return dImage;
  }

  //构建可拖拽控件
  _buildDraggable(BuildContext context) {
    var inContext = context;
    if (null == inContext) return;
    if (null == _showView) return;

    return new Draggable(
      child: _showView,
      feedback: _showView,
      onDragStarted: () {},
      onDragEnd: (detail) {
        LogUtil.e('onDragEnd:${detail.offset}');
        _endOffset = detail.offset;
        createDraggable(offset: detail.offset, pContext: inContext);
      },
      onDragCompleted: () {
        LogUtil.e("onDragCompleted");
      },
      onDraggableCanceled: (velocity, offset) {
        LogUtil.e(
            "onDraggableCanceled   velocity = ${velocity} , offset = ${offset}");
      },
      childWhenDragging: Container(),
    );
  }

  void refresh() {
    _holder?.markNeedsBuild();
  }

  //创建新的悬浮实例
  void createDraggable({Offset offset, BuildContext pContext}) {
    var inContext = pContext ?? widget.context;
    var _offset = offset ?? _endOffset;
    if (null == this._mData) return;
    if (null == inContext) return;
    if (null == _offset) return;
    remove();

    //创建新的悬浮实例
    _holder = new OverlayEntry(builder: (inContext) {
      double maxY = MediaQuery.of(inContext).size.height - 100;
      double maxX = MediaQuery.of(inContext).size.width - 90;

      LogUtil.e("maxX = ${MediaQuery.of(inContext).size.width} \r\n"
          "maxY = ${MediaQuery.of(inContext).size.height} \r\n");
      return new Positioned(
        top: _offset.dy < 50
            ? 50
            : _offset.dy < maxY
                ? offset.dy
                : maxY,
        left: _offset.dx < 0
            ? 0
            : _offset.dx > maxX
                ? maxX
                : offset.dx,
        child: _buildDraggable(inContext),
      );
    });

    try {
      Overlay.of(inContext).insert(_holder);
    } catch (e) {
      LogUtil.e("添加失败 ${e.toString()}  Context = ${inContext}");
    }
  }

  //拿数据
  Future<ResultBean<AdBean>> _requestData() async {
    var adSlot = widget.slotId;
    var userId = widget.userId;
    var params = {
      "deviceSlotExpCount": "1",
      "slotAccessType": "0",
      "slotSceneType": SDKType.WALL_VIEW,
    };

    ResultBean<AdBean> result = await AdApi.instance.loadAd(
        adSlot: adSlot,
        userId: userId,
        param: params,
        onLoadFailedCallBack: widget.onLoadFailedCallBack);
    return result;
  }

  //按钮点击
  _adClick(BuildContext pContext, ResultBean<AdBean> data) async {
    CommonUtils.banMultiClick(funFuture: () {
      if (widget.onClickCallBack != null) {
        widget.onClickCallBack();
      }
      _openActivity(pContext, data);
    });
  }

  //打开活动
  _openActivity(BuildContext pContext, ResultBean<AdBean> data) async {
    var inContext = pContext;
    if (null == inContext) return;

    //点击上报
    if (!_isClicked) {
      _reportData(1);
      _isClicked = true;
    }

    var result = await Navigator.push(
        inContext,
        MaterialPageRoute(
          builder: (inContext) => ActivityWebPage(
            data: data,
            entry: _holder,
          ),
        ));

    if (result.toString() == "退出") {
      debugPrint("活动页关闭");
      if (widget.onAdActivityCloseCallBack != null) {
        widget.onAdActivityCloseCallBack("");
      }
      if (widget.isDraggable) {
        createDraggable(offset: _endOffset, pContext: inContext);
      }
    }
  }

  //设置是否显示
  void setVisibility({bool isVisibility, BuildContext pContext}) {
    var inContext = pContext ?? widget.context;
    if (null == inContext) return;

    try {
      setState(() {
        _isVisibility = false;
      });
    } catch (e) {
      CrashUtils.crashReport(e);
    }

    if (isVisibility) {
      createDraggable(offset: _endOffset, pContext: inContext);
    } else {
      remove();
    }
  }

// params.put("slotAccessType","0");
// params.put("actualInteractiveType",String.valueOf(mActualInteractiveType));
// FoxViewMaidianUtils.doResponse(mAdslotId,logType,mData,params);
  //曝光&点击 reportType ： 0-曝光 1-点击
  void _reportData(int reportType) {
    var param = {"slotAccessType": "0", "actualInteractiveType": "0"};
    EnterApi.instance.reportData(reportType, _mData, param);
  }
}
