import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:sdk_flutter/global/cons/SDKType.dart';
import 'package:sdk_flutter/http/api/AdApi.dart';
import 'package:sdk_flutter/http/api/EnterApi.dart';
import 'package:sdk_flutter/model/AdBean.dart';
import 'package:sdk_flutter/model/ResultBean.dart';
import 'package:sdk_flutter/plugin/DecorationPlugin.dart';
import 'package:sdk_flutter/util/CommonUtils.dart';
import 'package:sdk_flutter/global/AdCallBack.dart';
import 'package:sdk_flutter/util/CrashUtils.dart';
import 'package:sdk_flutter/util/ImageUtils.dart';

import 'ActivityWebPage.dart';
import 'AdIcon.dart';

class InfoStreamWidget extends StatefulWidget {
  const InfoStreamWidget({
    @required this.context,
    @required this.slotId,
    @required this.userId,
    this.width = 0.0, //宽
    this.height = 120.0, //高
    this.imageWidth = 120,
    this.imageHeight = 120,
    this.type = 1,
    this.interval = 10, //文字图片间隔
    this.animation, //动画
    this.decoration, //样式 (暂不支持)
    this.isRounded = false,
    this.isShow = true, //是否显示
    this.radius = 0.0, //圆角度
    this.boxFit = BoxFit.fill, //图片填充方式
    this.onClickCallBack,
    this.onLoadSuccessCallBack,
    this.onAdCloseCallBack,
    this.onLoadFailedCallBack,
    this.onAdActivityCloseCallBack,
  }) : assert(context != null);

  final BuildContext context;
  final String slotId;
  final String userId;
  final double width;
  final double height;
  final double imageWidth;
  final double imageHeight;
  final int type;
  final double interval;
  final Animation animation; //动画
  final Decoration decoration; //样式 (暂不支持)
  final bool isRounded; //是否为圆角
  final bool isShow; //是否显示
  final double radius; //圆角度
  final BoxFit boxFit; //图片填充方式
  final AdClickCallBack onClickCallBack;
  final LoadSuccessCallBack onLoadSuccessCallBack;
  final AdCloseCallBack onAdCloseCallBack;
  final LoadFailedCallBack onLoadFailedCallBack;
  final AdActivityCloseCallBack onAdActivityCloseCallBack;

  @override
  _InfoStreamWidgetState createState() => _InfoStreamWidgetState();
}

class _InfoStreamWidgetState extends State<InfoStreamWidget> {
  Widget _showView;
  ResultBean<AdBean> _mData;
  BuildContext _mContext;
  bool _isVisibility = true;
  double _showWidth;
  bool _isClicked = false;

  @override
  Widget build(BuildContext context) {
    if (!_isVisibility) {
      _isVisibility = true;
      return Container(
        width: 0.0,
        height: 0.0,
      );
    }
    _mContext = context;
    _showWidth = widget.width <= 0
        ? (MediaQuery.of(_mContext).size.width)
        : widget.width;

    CommonUtils.getLastDateTime();
    _showView = FutureBuilder(
      future: _requestData(),
      builder: (context, snapshot) {
        switch (snapshot.connectionState) {
          case ConnectionState.none:
            debugPrint("---------ConnectionState.none----------");
            break;
          case ConnectionState.active:
            debugPrint("---------ConnectionState.active----------");
            break;
          case ConnectionState.waiting:
            debugPrint("---------ConnectionState.waiting----------");
            break;
          case ConnectionState.done:
            debugPrint("---------ConnectionState.done----------");
            if (snapshot.hasData) {
              // 通过数据构建界面
              _mData = snapshot.data;
              _isClicked = false;
              debugPrint("图片: ${_mData?.data?.imageUrl}");
              return getShowWidget(context);
            } else {
              // 显示加载Loading，直到数据返回时停止
              debugPrint('构建加载Loading');
              return Container(); //CircularProgressIndicator();
            }
            break;
        }
        return Container();
      },
    );

    return _showView;
  }

  //获取显示的Widget
  Widget getShowWidget(BuildContext context) {
    var iContext = context;
    if (null == iContext) return Container();
    if (null == _mData) return Container();

    debugPrint("屏幕宽度 = ${MediaQuery.of(_mContext).size.width}");
    //原型
    // var bannerView = Container(
    //   width: widget.width <= 0
    //       ? MediaQuery.of(_mContext).size.width
    //       : widget.width,
    //   height: widget.height,
    //   decoration: _getDecoration(_mData?.data?.imageUrl),
    //   child: MaterialButton(
    //     onPressed: () {
    //       _adClick(iContext, _mData);
    //     },
    //   ),
    // );

    var specType = _mData?.data?.specType;
    var cView = Stack(
      alignment: Alignment.centerRight,
      children: [
        getItem(specType),
        Container(
            width: _showWidth,
            height: widget.height,
            child: MaterialButton(
              onPressed: () {
                _adClick(iContext, _mData);
              },
            )),
        AdIcon.getAdIcon(
            _mData?.data?.isVisibleOfIcon,
            _mData?.data?.isVisibleOfCloseButton,
            widget.height,
            iconWidth: 28,
            iconHeight: 15,
            closeWidth: 15,
            closeHeight: 15, adCloseCallBack: () {
          setVisibility(isVisibility: false, pContext: iContext);
          if (null != widget.onAdCloseCallBack) {
            widget.onAdCloseCallBack();
          }
        }),
      ],
    );

    var rView = Visibility(
      child: cView,
      visible: _isVisibility,
    );

    this._showView = rView; //显示Widget

    return _showView;
  }

  //提供默认的Decoration
  Decoration _getDecoration(String imageUrl) {
    if (CommonUtils.isEmpty(imageUrl)) {
      return null;
    }
    Decoration dImage = DecorationPlugin.getCommonDecoration(
        dImage: ImageUtils.getNetworkImage(imageUrl, fit: BoxFit.fill,
            error: (exception, stackTrace) {
          debugPrint(
              "加载错误 exception = ${exception.toString()} stackTrace = ${stackTrace.toString()}");
          setVisibility(isVisibility: false, pContext: context);
        }),
        radius: widget.radius);
    if (widget.onLoadSuccessCallBack != null) {
      widget.onLoadSuccessCallBack();
    }
    return dImage;
  }

  //拿数据
  Future<ResultBean<AdBean>> _requestData() async {
    var adSlot = widget.slotId;
    var userId = widget.userId;

    var params = {
      "deviceSlotExpCount": "1",
      "slotAccessType": "0",
      "specTemplateFlag": "1",// 用与区分SDK新老版本（传1可以请求到四种模板类型）
      "isSupportApk": "1",
      "slotSceneType": SDKType.TEMPLET_INFO_FEED,
    };

    ResultBean<AdBean> result = await AdApi.instance.loadAd(
        adSlot: adSlot,
        userId: userId,
        param: params,
        onLoadFailedCallBack: widget.onLoadFailedCallBack);
    return result;
  }

  //按钮点击
  _adClick(BuildContext pContext, ResultBean<AdBean> data) async {
    CommonUtils.banMultiClick(funFuture: () {
      if (widget.onClickCallBack != null) {
        widget.onClickCallBack();
      }
      _openActivity(pContext, data);
    });
  }

  //打开活动
  _openActivity(BuildContext pContext, ResultBean<AdBean> data) async {
    var inContext = pContext;
    if (null == inContext) return;

    //点击上报
    if (!_isClicked) {
      _reportData(1);
      _isClicked = true;
    }

    var result = await Navigator.push(
        inContext,
        MaterialPageRoute(
          builder: (inContext) => ActivityWebPage(
            data: data,
          ),
        ));

    if (result.toString() == "退出") {
      if (widget.onAdActivityCloseCallBack != null) {
        widget.onAdActivityCloseCallBack("");
      }
      // createDraggable(offset: _endOffset, pContext: inContext);
    }
  }

  //设置是否显示
  void setVisibility({bool isVisibility, BuildContext pContext}) {
    var inContext = pContext ?? _mContext;
    if (null == inContext) return;

    try {
      setState(() {
        debugPrint(
            "隐藏Icon==========================================================");
        _isVisibility = false;
      });
    } catch (e) {
      CrashUtils.crashReport(e);
    }

    if (isVisibility) {
      // createDraggable(offset: _endOffset, pContext: inContext);
    } else {
      remove();
    }
  }

  void remove() {
    try {
      // _holder?.remove();
      // _holder = null;
    } catch (e) {}
  }

  void _reportData(int reportType) {
    var param = {"slotAccessType": "0", "actualInteractiveType": "0"};
    EnterApi.instance.reportData(reportType, _mData, param);
  }

  Widget getItem(int type) {
    var showItem;
    switch (type) {
      case 1:
        {
          //左文右图
          var textImage = Container(
            width: _showWidth,
            height: widget.height,
            child: Column(
              children: [
                Row(
                  children: [
                    //文字内容
                    Container(
                      width: _showWidth - widget.imageWidth - widget.interval,
                      height: widget.height,
                      child: Text(
                        _mData?.data?.extTitle.toString(),
                        overflow: TextOverflow.ellipsis,
                        maxLines: 6,
                        softWrap: true,
                      ),
                    ),
                    Spacer(),
                    getImageWidget(_mData?.data?.imageUrl, widget.imageWidth,
                        widget.imageHeight)
                  ],
                ),
                Container()
              ],
            ),
          );
          showItem = textImage;
          break;
        }
      case 3:
        {
          //右文左图
          var imageText = Container(
            width: _showWidth,
            height: widget.height,
            child: Column(
              children: [
                Row(
                  children: [
                    getImageWidget(_mData?.data?.imageUrl, widget.imageWidth,
                        widget.imageHeight),
                    Spacer(),
                    //文字内容
                    Container(
                      width: _showWidth - widget.imageWidth - widget.interval,
                      height: widget.height,
                      child: Text(
                        _mData?.data?.extTitle.toString(),
                        overflow: TextOverflow.ellipsis,
                        maxLines: 6,
                        softWrap: true,
                      ),
                    ),
                  ],
                ),
                Container()
              ],
            ),
          );
          showItem = imageText;
          break;
        }
      case 2:
        {
          //上文下图
          var largeImage = Container(
            width: _showWidth,
            height: widget.height,
            child: Column(
              children: [
                Container(
                  width: _showWidth,
                  height: 20,
                  child: Text(
                    _mData?.data?.extTitle.toString(),
                    overflow: TextOverflow.ellipsis,
                    maxLines: 1,
                    softWrap: true,
                  ),
                ),
                Spacer(),
                getImageWidget(_mData?.data?.imageUrl, _showWidth,
                    widget.imageHeight - 20),
              ],
            ),
          );
          showItem = largeImage;
          break;
        }
      case 4:
        {
          var groupImage = Container(
            width: _showWidth,
            height: widget.height,
            child: Column(
              children: [
                Container(
                  width: _showWidth,
                  height: 20,
                  child: Text(
                    _mData?.data?.extTitle.toString(),
                    overflow: TextOverflow.ellipsis,
                    maxLines: 1,
                    softWrap: true,
                  ),
                ),
                Spacer(),
                Flex(
                  direction: Axis.horizontal,
                  mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                  children: [
                    getImageWidget(_mData?.data?.imageUrlList[0], _showWidth / 3 - 25,
                        _showWidth / 3 - 25),
                    getImageWidget(_mData?.data?.imageUrlList[1], _showWidth / 3 - 25,
                        _showWidth / 3 - 25),
                    getImageWidget(_mData?.data?.imageUrlList[2], _showWidth / 3 - 25,
                        _showWidth / 3 - 25)
                  ],
                ),
              ],
            ),
          );
          showItem = groupImage;
          break;
        }
    }
    _reportData(0);
    return showItem;
  }

  Widget getImageWidget(String imageUrl, double width, double height) {
    var imageWidget = Container(
      width: width,
      height: height,
      decoration: _getDecoration(imageUrl),
    );

    return imageWidget;
  }

  @override
  void initState() {
    super.initState();
  }
}
