import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:sdk_flutter/global/cons/SDKType.dart';
import 'package:sdk_flutter/http/api/AdApi.dart';
import 'package:sdk_flutter/http/api/EnterApi.dart';
import 'package:sdk_flutter/model/AdBean.dart';
import 'package:sdk_flutter/model/ResultBean.dart';
import 'package:sdk_flutter/plugin/DecorationPlugin.dart';
import 'package:sdk_flutter/util/CommonUtils.dart';
import 'package:sdk_flutter/global/AdCallBack.dart';
import 'package:sdk_flutter/util/CrashUtils.dart';
import 'package:sdk_flutter/util/ImageUtils.dart';

import 'ActivityWebPage.dart';
import 'AdIcon.dart';

class PopupWidget extends StatefulWidget {
  const PopupWidget({
    @required this.context,
    @required this.slotId,
    @required this.userId,
    this.width = 200.0, //宽
    this.height = 200.0, //高
    this.animation, //动画
    this.decoration, //样式 (暂不支持)
    this.isRounded = false,
    this.isShow = true, //是否显示
    this.radius = 0.0, //圆角度
    this.boxFit = BoxFit.fill, //图片填充方式
    this.onClickCallBack,
    this.onLoadSuccessCallBack,
    this.onAdCloseCallBack,
    this.onLoadFailedCallBack,
    this.onAdActivityCloseCallBack,
  }) : assert(context != null);

  final BuildContext context;
  final String slotId;
  final String userId;
  final double width;
  final double height;
  final Animation animation; //动画
  final Decoration decoration; //样式 (暂不支持)
  final bool isRounded; //是否为圆角
  final bool isShow; //是否显示
  final double radius; //圆角度
  final BoxFit boxFit; //图片填充方式
  final AdClickCallBack onClickCallBack;
  final LoadSuccessCallBack onLoadSuccessCallBack;
  final AdCloseCallBack onAdCloseCallBack;
  final LoadFailedCallBack onLoadFailedCallBack;
  final AdActivityCloseCallBack onAdActivityCloseCallBack;

  @override
  _PopupWidgetState createState() => _PopupWidgetState();
}

class _PopupWidgetState extends State<PopupWidget> {
  Widget _showView;
  ResultBean<AdBean> _mData;
  BuildContext _mContext;
  bool _isVisibility = true;
  bool _isClicked = false;

  @override
  Widget build(BuildContext context) {
    _mContext = context;

    CommonUtils.getLastDateTime();
    _showView = FutureBuilder(
      future: _requestData(),
      builder: (context, snapshot) {
        switch (snapshot.connectionState) {
          case ConnectionState.none:
            debugPrint("---------ConnectionState.none----------");
            break;
          case ConnectionState.active:
            debugPrint("---------ConnectionState.active----------");
            break;
          case ConnectionState.waiting:
            debugPrint("---------ConnectionState.waiting----------");
            break;
          case ConnectionState.done:
            debugPrint("---------ConnectionState.done----------");
            if (snapshot.hasData) {
              // 通过数据构建界面
              _mData = snapshot.data;
              _isClicked = false;
              debugPrint("图片: ${_mData?.data?.imageUrl}");
              return getShowWidget(context);
            } else {
              // 显示加载Loading，直到数据返回时停止
              // debugPrint('构建加载Loading');
              remove();
              return Container(); //CircularProgressIndicator();
            }
            break;
        }
        return Container();
      },
    );

    // 半透明蒙层
    var maskView = Container(
      width: double.infinity,
      height: double.infinity,
      color: Color(0x33000000),
      alignment: Alignment.center,
      child: GestureDetector(
        onTap: () {
          remove();
          if (null != widget.onAdCloseCallBack) {
            widget.onAdCloseCallBack();
          }
        },
      ),
    );

    var finalView = Stack(
      alignment: Alignment.center,
      children: [
        maskView,
        _showView,
      ],
    );

    return finalView;
  }

  //获取显示的Widget
  Widget getShowWidget(BuildContext context) {
    var iContext = context;
    if (null == iContext) return Container();
    if (null == _mData) return Container();

    debugPrint("屏幕宽度 = ${MediaQuery.of(_mContext).size.width}");
    //原型
    var popupView = Container(
      width: widget.width <= 0
          ? MediaQuery.of(_mContext).size.width
          : widget.width,
      height: widget.width <= 0
          ? MediaQuery.of(_mContext).size.height
          : widget.height,
      decoration: _getDecoration(_mData?.data?.imageUrl),
      child: MaterialButton(
        onPressed: () {
          _adClick(iContext, _mData);
        },
      ),
    );

    var cView = Stack(
      alignment: Alignment.centerRight,
      children: [
        popupView,
        AdIcon.getAdIcon(
            _mData?.data?.isVisibleOfIcon,
            _mData?.data?.isVisibleOfCloseButton,
            widget.height,
            iconWidth: 28,
            iconHeight: 15,
            closeWidth: 20,
            closeHeight: 20, adCloseCallBack: () {
          remove();
          if (null != widget.onAdCloseCallBack) {
            widget.onAdCloseCallBack();
          }
        }),
      ],
    );

    var adView = Visibility(
      child: cView,
      visible: _isVisibility,
    );

    return adView;
  }

  //提供默认的Decoration
  Decoration _getDecoration(String imageUrl) {
    if (CommonUtils.isEmpty(imageUrl)) {
      return null;
    }
    if (widget.onLoadSuccessCallBack != null) {
      widget.onLoadSuccessCallBack();
    }
    Decoration dImage = DecorationPlugin.getCommonDecoration(
        dImage: ImageUtils.getNetworkImage(imageUrl, fit: BoxFit.fill,
            error: (exception, stackTrace) {
          debugPrint(
              "图片加载错误 exception = ${exception.toString()} stackTrace = ${stackTrace.toString()}");
          setVisibility(isVisibility: false, pContext: context);
        }),
        radius: widget.radius);
    _reportData(0);
    return dImage;
  }

  //拿数据
  Future<ResultBean<AdBean>> _requestData() async {
    var adSlot = widget.slotId;
    var userId = widget.userId;

    var params = {
      "deviceSlotExpCount": "1",
      "slotAccessType": "0",
      "slotSceneType": SDKType.TB_SCREEN,
    };

    ResultBean<AdBean> result = await AdApi.instance.loadAd(
        adSlot: adSlot,
        userId: userId,
        param: params,
        onLoadFailedCallBack: widget.onLoadFailedCallBack);
    return result;
  }

  //按钮点击
  _adClick(BuildContext pContext, ResultBean<AdBean> data) async {
    CommonUtils.banMultiClick(funFuture: () {
      if (widget.onClickCallBack != null) {
        widget.onClickCallBack();
      }
      _openActivity(pContext, data);
    });
  }

  //打开活动
  _openActivity(BuildContext pContext, ResultBean<AdBean> data) async {
    var inContext = pContext;
    if (null == inContext) return;

    //点击上报
    if (!_isClicked) {
      _reportData(1);
      _isClicked = true;
    }

    var result = await Navigator.push(
        inContext,
        MaterialPageRoute(
          builder: (inContext) => ActivityWebPage(
            data: data,
          ),
        ));

    if (result.toString() == "退出") {
      debugPrint("活动页关闭");
      if (widget.onAdActivityCloseCallBack != null) {
        widget.onAdActivityCloseCallBack("");
      }
    }
  }

  //设置是否显示
  void setVisibility({bool isVisibility, BuildContext pContext}) {
    var inContext = pContext ?? _mContext;
    if (null == inContext) return;

    try {
      setState(() {
        _isVisibility = false;
      });
    } catch (e) {
      CrashUtils.crashReport(e);
    }

    if (isVisibility) {
      // createDraggable(offset: _endOffset, pContext: inContext);
    } else {
      remove();
    }
  }

  void remove() {
    try {
      Navigator.pop(context);
    } catch (e) {
      CrashUtils.crashReport(e);
    }
  }

  void _reportData(int reportType) {
    var param = {"slotAccessType": "0", "actualInteractiveType": "0"};
    EnterApi.instance.reportData(reportType, _mData, param);
  }
}
