import 'package:flutter/material.dart';
import 'package:sdk_flutter/global/AdCallBack.dart';
import 'package:sdk_flutter/global/cons/SDKType.dart';
import 'package:sdk_flutter/http/api/AdApi.dart';
import 'package:sdk_flutter/http/api/EnterApi.dart';
import 'package:sdk_flutter/model/AdBean.dart';
import 'package:sdk_flutter/model/ResultBean.dart';
import 'package:sdk_flutter/plugin/CountDownPlugin.dart';
import 'package:sdk_flutter/plugin/DecorationPlugin.dart';
import 'package:sdk_flutter/util/CommonUtils.dart';

import 'ActivityWebPage.dart';

class SplashAd {
  String _mSlotId;
  String _mUserId;
  String _mUrl;

  BuildContext _mContext;
  Image _mImage;

  ResultBean<AdBean> _mData;

  AdClickCallBack _onClickCallBack;
  LoadSuccessCallBack _onLoadSuccessCallBack;
  AdCloseCallBack _onAdCloseCallBack;
  LoadFailedCallBack _onLoadFailedCallBack;
  AdActivityCloseCallBack _onAdActivityCloseCallBack;

  CountDownPlugin countdown;

  //显示素材
  Widget loadAd({
    @required BuildContext pContext,
    @required String slotId,
    @required String userId,
    String url,
    Image image,
    AdClickCallBack onClickCallBack,
    LoadSuccessCallBack onLoadSuccessCallBack,
    AdCloseCallBack onAdCloseCallBack,
    LoadFailedCallBack onLoadFailedCallBack,
    AdActivityCloseCallBack onAdActivityCloseCallBack,
  }) {
    _mContext = pContext;
    _mSlotId = slotId;
    _mUserId = userId;
    _mUrl = url;
    _mImage = image;
    _onClickCallBack = onClickCallBack;
    _onLoadSuccessCallBack = onLoadSuccessCallBack;
    _onAdCloseCallBack = onAdCloseCallBack;
    _onLoadFailedCallBack = onLoadFailedCallBack;
    _onAdActivityCloseCallBack = onAdActivityCloseCallBack;

    countdown = CountDownPlugin(_onAdCloseCallBack);

    return FutureBuilder(
      future: _requestData(),
      builder: (context, snapshot) {
        if (snapshot.hasData) {
          debugPrint('构建Widget: ${snapshot.data}');
          // 通过数据构建界面
          _mData = snapshot.data;
          debugPrint("图片: ${_mData?.data?.imageUrl}");
          return getShowWidget(context, _mData.data.sckId);
        } else {
          // 显示加载Loading，直到数据返回时停止
          debugPrint('构建加载Loading');
          return Container(); //CircularProgressIndicator();
        }
      },
    );
  }

  //拿数据
  Future<ResultBean<AdBean>> _requestData() async {
    var adSlot = _mSlotId;
    var userId = _mUserId;

    var params = {
      "deviceSlotExpCount": "1",
      "slotAccessType": "0",
      "slotSceneType": SDKType.NATIVE_SPLASH,
    };

    ResultBean<AdBean> result = await AdApi.instance.loadAd(
        adSlot: adSlot,
        userId: userId,
        param: params,
        onLoadFailedCallBack: _onLoadFailedCallBack);
    return result;
  }

  //获取显示的Widget
  Widget getShowWidget(BuildContext context, int sckId) {
    var iContext = context;
    if (null == iContext) return Container();
    if (null == _mData) return Container();

    //原型
    return Stack(
      alignment: Alignment.topLeft,
      children: <Widget>[
        Container(
          width: double.infinity,
          height: double.infinity,
          decoration: sckId == 0
              ? (_mImage != null
                  ? _mImage
                  : _getDecoration(
                      _mUrl != null ? _mUrl : _mData?.data?.imageUrl))
              : _getDecoration(_mData?.data?.imageUrl),
          child: MaterialButton(
            onPressed: () {
              _adClick(iContext, _mData);
            },
          ),
        ),
        Container(
          alignment: Alignment(-1.0, 1.0),
          margin: EdgeInsets.only(left: 10.0, bottom: 5.0),
          width: 30.0,
          child: _mData.data.isVisibleOfIcon
              ? Image(
                  image: AssetImage('assets/images/ad.png',
                      package: "sdk_flutter"))
              : Container(),
        ),
        Container(
          alignment: Alignment.topRight,
          margin: new EdgeInsets.all(10.0),
          child: countdown,
        )
      ],
    );
  }

  //提供默认的Decoration
  Decoration _getDecoration(String imageUrl) {
    if (imageUrl.isEmpty) {
      return null;
    }
    if (_onLoadSuccessCallBack != null) {
      _onLoadSuccessCallBack();
    }

    _reportData(0);

    return DecorationPlugin.getImage(
        dImage:
            DecorationImage(image: NetworkImage(imageUrl), fit: BoxFit.fill));
  }

  //按钮点击
  _adClick(BuildContext pContext, ResultBean<AdBean> data) async {
    CommonUtils.banMultiClick(funFuture: () {
      if (_onClickCallBack != null) {
        _onClickCallBack();
      }
      _openActivity(pContext, data);
    });
  }

  //打开活动
  _openActivity(BuildContext pContext, ResultBean<AdBean> data) async {
    var inContext = pContext;
    if (null == inContext) return;

    //点击上报
    _reportData(1);
    countdown.autoClose(false);

    //TODO 点击上报
    String clickUrl = data.data.reportClickUrl;

    Navigator.pop(inContext);

    var result = await Navigator.push(
        inContext,
        MaterialPageRoute(
          builder: (inContext) => ActivityWebPage(
            data: data,
          ),
        ));

    if (result.toString() == "退出") {
      debugPrint("活动页关闭");
      if (_onAdActivityCloseCallBack != null) {
        _onAdActivityCloseCallBack("");
      }
    }
  }

  // params.put("slotAccessType","0");
  // params.put("actualInteractiveType",String.valueOf(mActualInteractiveType));
  // FoxViewMaidianUtils.doResponse(mAdslotId,logType,mData,params);
  //曝光&点击 reportType ： 0-曝光 1-点击
  void _reportData(int reportType) {
    var param = {"slotAccessType": "0", "actualInteractiveType": "0"};
    EnterApi.instance.reportData(reportType, _mData, param);
  }
}
