import 'package:flutter/material.dart';
import 'package:sdk_flutter/global/CommonConfig.dart';
import 'package:sdk_flutter/global/TFConfig.dart';
import 'package:sdk_flutter/http/http_utils/HttpUtil.dart';

import '../sdk_flutter.dart';
import 'cons/CommonData.dart';
import 'cons/ConsUrl.dart';
import 'dart:convert' as convert;

class CheckData {
  factory CheckData() => _getInstance();

  CheckData._private();

  static CheckData _instance;

  static CheckData get instance => _getInstance();

  static _getInstance() {
    if (null == _instance) {
      _instance = CheckData._private();
    }
    return _instance;
  }

  void startCheckData(String unCollects) async {
    final appSecret = TFConfig.appSecret;
    final appKey = TFConfig.appKey;
    final amassMd = await SdkFlutter.getAmassMD(appKey,unCollects,CommonData.SDK_VERSION);
    final nonce = await SdkFlutter.nonce;
    final timestamp = await SdkFlutter.timestamp;
    final signature =
        await SdkFlutter.getSignature(appSecret, nonce, timestamp,CommonData.SDK_VERSION);
    var signMap = {
      "time": DateTime.now().millisecondsSinceEpoch,
      "appKey": appKey,
      "nonce": nonce,
      "signature": signature,
      "md": amassMd
    };
    final sign = await SdkFlutter.getSign(signMap, CommonConfig.getInstance().hostKey);
    final appName = await SdkFlutter.appName;
    final packageName = await SdkFlutter.packageName;
    final sdkVersion = CommonData.SDK_VERSION;
    var params = {
      "sign": sign,
      "app_name": appName,
      "app_key": appKey,
      "sdk_version": sdkVersion,
      "package_name": packageName,
    };
    String dataJson = convert.jsonEncode(params);
    var response = await HttpUtil.getInstance()
        .client
        .post(ConsUrl.AMASS, data: dataJson)
        .catchError((e) {
      debugPrint("请求错误 ${e.toString()}");
    });
    debugPrint("实际响应 ===" + response.toString());
  }
}
