import 'package:flutter/material.dart';
import 'package:sdk_flutter/global/CommonConfig.dart';
import 'package:sdk_flutter/global/TFConfig.dart';
import 'package:sdk_flutter/http/http_utils/HttpUtil.dart';

import '../sdk_flutter.dart';
import 'cons/CommonData.dart';
import 'cons/ConsUrl.dart';
import 'dart:convert' as convert;

class CrashReport {
  factory CrashReport() => _getInstance();

  CrashReport._private();

  static CrashReport _instance;

  static CrashReport get instance => _getInstance();

  static _getInstance() {
    if (null == _instance) {
      _instance = CrashReport._private();
    }
    return _instance;
  }

  void crashReport(String crashLog,String dataType, String crashType) async {

    final appKey = TFConfig.appKey;
    final appSecret = TFConfig.appSecret;
    final nonce = await SdkFlutter.nonce;
    final timestamp = await SdkFlutter.timestamp;
    final signature = await SdkFlutter.getSignature(appSecret, nonce, timestamp,CommonData.SDK_VERSION);
    final crashMd = await SdkFlutter.getCrashMd(appKey, CommonData.SDK_VERSION, crashLog,dataType,crashType);
    var signMap = {
      "time": DateTime.now().millisecondsSinceEpoch,
      "appKey": appKey,
      "nonce": nonce,
      "signature": signature,
      "md": crashMd
    };
    final sign = await SdkFlutter.getSign(signMap, CommonConfig.getInstance().hostKey);
    var params = {
      "sign": sign,
    };

    String dataJson = convert.jsonEncode(params);
    var response = await HttpUtil.getInstance()
        .client
        .post(ConsUrl.REPORT_CRASH, data: dataJson)
        .catchError((e) {
      debugPrint("请求错误 ${e.toString()}");
    });
    debugPrint("实际响应 ===" + response.toString());
  }
}
