import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:sdk_flutter/global/TFConfig.dart';
import 'package:sdk_flutter/http/http_utils/HttpUtil.dart';
import 'package:sdk_flutter/model/CommonConfigBean.dart';
import 'package:sdk_flutter/model/ResultBean.dart';
import 'package:sdk_flutter/util/CommonUtils.dart';
import 'package:sdk_flutter/util/CrashUtils.dart';
import 'package:shared_preferences/shared_preferences.dart';

import '../sdk_flutter.dart';
import 'CheckData.dart';
import 'cons/CommonData.dart';
import 'cons/ConsUrl.dart';
import 'dart:convert' as convert;

class QueryCommonConfig {
  factory QueryCommonConfig() => _getInstance();

  QueryCommonConfig._private();

  static QueryCommonConfig _instance;

  static QueryCommonConfig get instance => _getInstance();

  static _getInstance() {
    if (null == _instance) {
      _instance = QueryCommonConfig._private();
    }
    return _instance;
  }

  CommonConfigBean mData;
  String mUnCollects;

  Future<ResultBean<CommonConfigBean>> queryCommonConfig() async {
    final appKey = await TFConfig.appKey;
    final imei = await SdkFlutter.imei;
    final deviceId = await SdkFlutter.deviceId;
    final tcid = await SdkFlutter.tcid;
    final phoneBrand = await SdkFlutter.phoneBrand;
    final phoneModel = await SdkFlutter.phoneModel;
    final oaid = await SdkFlutter.oaid;
    final sdkVersion = CommonData.SDK_VERSION;
    final osType = await SdkFlutter.osType;
    final idfa = await SdkFlutter.idfa;
    var sha1Map = {
      "appKey": appKey,
      "phoneBrand": phoneBrand,
      "phoneModel": phoneModel,
      "imei": imei,
      "oaid": oaid,
      "idfa": idfa,
      "deviceId": deviceId,
      "sdkVersion": sdkVersion,
      "osType": osType,
      "patchVersion": "0.0",
      "patchCode": "0",
    };
    final sha1 = await SdkFlutter.getSha1(sha1Map);
    sha1Map["signature"] = sha1;//"1c134591ea572ec723debeb1fe761cc57818fdeb";//1c134591ea572ec723debeb1fe761cc57818fdeb
    sha1Map["tcid"] = tcid;

    String dataJson = convert.jsonEncode(sha1Map);
    var response = await HttpUtil.getInstance()
        .client
        .post(ConsUrl.QUERY_COMMON_CONFIG, data: dataJson)
        .catchError((e) {
      debugPrint("请求错误 ${e.toString()}");
      CrashUtils.crashReport(e,isReport: false);
    });
    debugPrint("实际响应 ===" + response.toString());
    if(response?.data == null || response?.data["data"] == "null"){
      return null;
    }
    return ResultBean<CommonConfigBean>(
        data: CommonConfigBean.fromJson(response.data['data']),
        code: response.data['code'],
        desc: response.data['desc'],
        success: response.data['success']);
  }

  void startQueryCommonConfig() async {
    ResultBean<CommonConfigBean> configBean =
        await QueryCommonConfig.instance.queryCommonConfig();

    if(configBean == null || configBean?.data == null){
      return;
    }
    mData = configBean.data;
    List<dynamic> unCollects = mData.collectShieldFields;
    if (!CommonUtils.isEmptyList(unCollects)) {
      mUnCollects = JsonEncoder().convert(unCollects);
    }
    if(mData.collectEnable == 1 && mData.isCollectUserInfo){
      debugPrint("开始数据采集");
      CheckData.instance.startCheckData(mUnCollects);
    }
    Future<SharedPreferences> _prefs = SharedPreferences.getInstance();
    final SharedPreferences prefs = await _prefs;
    //Crash上报开关
    prefs.setInt(CommonData.KEY_TUIA_SDK_ERROR_REPORT_SWITCH, mData.flutterCrashLogEnable);

  }
}
