import 'package:flutter/material.dart';
import 'package:sdk_flutter/global/AdCallBack.dart';
import 'package:sdk_flutter/global/CommonConfig.dart';
import 'package:sdk_flutter/global/cons/CommonData.dart';
import 'package:sdk_flutter/global/cons/ConsUrl.dart';
import 'package:sdk_flutter/global/cons/SDKType.dart';
import 'package:sdk_flutter/http/http_utils/HttpUtil.dart';
import 'package:sdk_flutter/model/AdBean.dart';
import 'package:sdk_flutter/model/ResultBean.dart';
import 'package:sdk_flutter/sdk_flutter.dart';
import 'package:sdk_flutter/util/CrashUtils.dart';

class AdApi {
  factory AdApi() => _getInstance();

  AdApi._private();

  static AdApi get instance => _getInstance();
  static AdApi _instance;

  static _getInstance() {
    if (_instance == null) {
      _instance = AdApi._private();
    }
    return _instance;
  }

  var mAppKey = CommonConfig.getInstance().appKey;
  var mAppSecret = CommonConfig.getInstance().appSecret;
  LoadFailedCallBack mOnLoadFailedCallBack;

  //TODO 修改AppKey和AppSecret

  void setConfig({@required String appKey, @required String appSecret}) {
    mAppKey = appKey;
    mAppSecret = appSecret;
  }

  //请求数据
  Future<ResultBean<AdBean>> loadAd(
      {@required String adSlot, String userId,Map<String,String> param,LoadFailedCallBack onLoadFailedCallBack}) async {
    mOnLoadFailedCallBack = onLoadFailedCallBack;
    return postAd(adSlot, userId.isEmpty ? "" : userId, param);
  }

  Future<ResultBean<AdBean>> postAd(String adSlot, String userId,Map<String,String> param) async {
    var response;
    final md = await SdkFlutter.getMd(CommonData.SDK_VERSION);
    final nonce = await SdkFlutter.nonce;
    final timestamp = await SdkFlutter.timestamp;
    final imei = await SdkFlutter.imei;
    final deviceId = await SdkFlutter.deviceId;
    final signature =
        await SdkFlutter.getSignature(mAppSecret, nonce, timestamp,CommonData.SDK_VERSION);
    final tcid = await SdkFlutter.tcid;
    var params = {
      "adslotId": adSlot,
      "appKey": mAppKey,
      "md": md,
      "timestamp": timestamp,
      "nonce": nonce,
      "signature": signature,
      "isimageUrl": "1", //是否使用图片
      "sourceType": "1",
      "device_id": deviceId,
      "userId": userId,
      "tcid": tcid,
      "fox_n": "", //TODO 待实现
      "preloading": "0"

    };
    if(null != param){
      param.forEach((key, value) {
        params[key] = value;
      });
    }
    // response = await BaseApi.instance.postAd(ConsUrl.BASE_URL, param);
    response = await HttpUtil.getInstance()
        .client
        .post(ConsUrl.BASE_URL, queryParameters: params)
        .catchError((e) {
      debugPrint("请求错误 ${e.toString()}");
      CrashUtils.crashReport(e, isReport: false);
      if(null != mOnLoadFailedCallBack){
        mOnLoadFailedCallBack(SDKType.NETWORK_ERROR[0],SDKType.NETWORK_ERROR[1]);
      }
    });
    //失败请求
    String code = response.data['code'];
    String desc = response.data['desc'];
    if(code != "0"){
      if(null != mOnLoadFailedCallBack){
        mOnLoadFailedCallBack(code,desc);
      }
    }
    // debugPrint("实际响应 === " + response.toString());
    return ResultBean<AdBean>(
        data: AdBean.fromJson(response.data['data']),
        code: response.data['code'],
        desc: response.data['desc'],
        success: response.data['success']);
  }
}
