import 'dart:io';

import 'package:dio/adapter.dart';
import 'package:dio/dio.dart';
import 'package:flutter/foundation.dart';
import 'package:sdk_flutter/global/CommonConfig.dart';
import 'package:sdk_flutter/global/cons/CommonData.dart';
import 'package:sdk_flutter/util/CommonUtils.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'LogsInterceptors.dart';
import 'TokenInterceptors.dart';

const int _kReceiveTimeout = 5000;
const int _kSendTimeout = 5000;
const int _kConnectTimeout = 5000;

class HttpUtil {
  static HttpUtil _instance = HttpUtil._internal();
  Dio _dio;

  static const CODE_SUCCESS = 200;
  static const CODE_TIME_OUT = -1;

  factory HttpUtil() => _instance;
  TokenInterceptors tokenInterceptors;

  ///通用全局单例，第一次使用时初始化
  HttpUtil._internal() {
    if (null == _dio) {
      _dio = Dio(BaseOptions(
        baseUrl: CommonConfig.getInstance().host,
        connectTimeout: _kReceiveTimeout,
        receiveTimeout: _kConnectTimeout,
        sendTimeout: _kSendTimeout,
      ));
      _dio.interceptors.add(LogsInterceptors());
      // _dio.interceptors.add(ResponseInterceptors());
      //设置代理
      proxyConfig();
    }
  }

  static HttpUtil getInstance() {
    return _instance._default();
  }

  Dio get client => _dio;

  void setToken(String token) {
    print('---token---$token-------');
    tokenInterceptors = TokenInterceptors(token: token);
    _dio.interceptors.add(tokenInterceptors);
  }

  void deleteToken() {
    _dio.interceptors.remove(tokenInterceptors);
  }

  void rebase(String baseIp) {
    _dio.options.baseUrl = baseIp;
  }

  //一般请求，默认域名
  HttpUtil _default() {
    if (_dio != null) {
      _dio.options.baseUrl = CommonConfig.getInstance().host;
      _dio.options.headers = {};
    }

    return this;
  }

  void proxyConfig() async {
    // if (!kReleaseMode) {
      Future<SharedPreferences> _prefs = SharedPreferences.getInstance();
      final SharedPreferences prefs = await _prefs;
      String ip = prefs.getString(CommonData.HTTP_PROXY_IP);
      if(CommonUtils.isEmpty(ip) ) return;

      (_dio.httpClientAdapter as DefaultHttpClientAdapter).onHttpClientCreate =
          (client) {
        //这一段是解决安卓https抓包的问题
        client.badCertificateCallback =
            (X509Certificate cert, String host, int port) {
          return Platform.isAndroid;
        };
        client.findProxy = (uri) {
          return "PROXY " + ip;
          // return "PROXY 172.16.74.123:8888";
        };
      };
    // }
  }
}
