import 'dart:io';

import 'package:dio/dio.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';
import 'package:install_plugin/install_plugin.dart';
import 'package:path_provider/path_provider.dart';
import 'package:sdk_flutter/util/CommonUtils.dart';

class DownloadPlugin {

  static int _lastProgress = 0;
  static double progress = 0.0;
  static String path;
  static bool showNotificationFlag = true;
  static Dio dio;

  static Future<Null> defaultDownload(@required String url, @required String packageName, @required String tuiaId) async{
    _download(url, packageName, tuiaId, true);
  }

  static Future<Null> backgroundDownload(@required String url, @required String packageName, @required String tuiaId) async{
    _download(url, packageName, tuiaId, false);
  }

  static cancelTask(){
    if(dio == null){
      return;
    }


  }

  static continueTask(){
    if(dio == null){
      return;
    }

    dio.unlock();
  }

  static Future<Null> showNotification(bool show) async{
    showNotificationFlag = show;
  }

  static double getProgress(@required String url, @required String packageName, @required String tuiaId) {
    return progress;
  }

  static Future<Null> installApp(@required String url, @required String packageName, @required tuiaId) async{
    _installApk(path);
  }

  static Future<Null> _download(String url, String packageName, String tuiaId, bool show) async {
    _registerBroadCast(url, packageName, tuiaId);
    showNotificationFlag = show;
    if(dio == null){
      dio = new Dio();
    }

    dio.clear();
    String dir = "";
    File apk;
    if(Platform.isAndroid){
      Directory directory = await getExternalStorageDirectory();
      dir = directory.path;
    }

    try{
      CommonUtils.createFolder(dir);
      apk = new File(dir + "/" + packageName + ".apk");
      if(!apk.existsSync()){
        apk.createSync();

        Response response = await dio.get(url, onReceiveProgress: (receiveBytes, totalBytes){
          progress = (receiveBytes / totalBytes) * 100;
          int p = progress.truncate();

          if(showNotificationFlag && _lastProgress != progress.truncate()){
            _updateBroadCast(dir + "/" + packageName + ".apk", packageName, p.toString(), tuiaId);
          }
          _lastProgress = progress.truncate();
        }, options: Options(
          responseType: ResponseType.bytes,
          followRedirects: false,
        ));
        apk.writeAsBytesSync(response.data);
      }

      path = apk.path;
      progress = 0;

      if(showNotificationFlag){
        _installApk(path);
      }
    }catch(e){
      print(e);
    }
  }

  // static Future _getContentLength(Dio dio, String url) async{
  //   try{
  //     Response response = await dio.head(url);
  //     return response.headers.
  //   }catch(e){
  //
  //   }
  // }

  static void _installApk(String apkPath) {
    if(apkPath.isEmpty){
      return;
    }

    InstallPlugin.installApk(apkPath, "com.mediamain.sdk_flutter_example")
        .then((result) {
      path = "";
    }).catchError((error) {
      print('install apk error: $error');
    });
  }

  static void _updateBroadCast(String path, String packageName, String p, String tuiaId){
    const platform = MethodChannel('sdk_flutter');
    platform.invokeMethod('installBroadCast', {'apkPath': path, 'packageName': packageName, 'progress': p, "tuiaId": tuiaId});
  }

  static void _registerBroadCast(String path, String packageName, String tuiaId){
    const platform = MethodChannel('sdk_flutter');
    platform.invokeMethod('registerBroadCast', {'apkPath': path, 'packageName': packageName, "tuiaId": tuiaId});
  }
}