import 'dart:async';

import 'package:flutter/services.dart';

class SdkFlutter {
  static const MethodChannel _channel = const MethodChannel('sdk_flutter');

  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('version');
    return version;
  }

  static Future<String> get imei async {
    final String imei = await _channel.invokeMethod('getImei');
    return imei;
  }

  static Future<String> get deviceId async {
    final String deviceId = await _channel.invokeMethod('getDeviceId');
    return deviceId;
  }

  static Future<String> getMd(String sdkVersion) async {
    final String md = await _channel.invokeMethod('getMD', <String, dynamic>{
      "sdkVersion": sdkVersion,
    });
    return md;
  }

  static Future<String> get nonce async {
    final String nonce = await _channel.invokeMethod('getNonce');
    return nonce;
  }

  static Future<String> get tcid async {
    final String nonce = await _channel.invokeMethod('getTcid');
    return nonce;
  }

  static Future<String> get timestamp async {
    final String timestamp = await _channel.invokeMethod('getTimestamp');
    return timestamp;
  }

  static Future<String> getSignature(String appSecret, String nonce,
      String timestamp, String sdkVersion) async {
    final String signature =
        await _channel.invokeMethod('getSignature', <String, dynamic>{
      "appSecret": appSecret,
      "nonce": nonce,
      "timestamp": timestamp,
      "sdkVersion": sdkVersion,
    });
    return signature;
  }

  static Future<String> getSign(Map maps, String hostKey) async {
    final String signature =
        await _channel.invokeMethod('getSign', <String, dynamic>{
      "maps": maps,
      "hostKey": hostKey,
    });
    return signature;
  }

  static Future<String> getSha1(Map sha1Map) async {
    final String signature =
        await _channel.invokeMethod('getSha1', <String, dynamic>{
      "sha1Map": sha1Map,
    });
    return signature;
  }

  static Future<String> get idfa async {
    final String idfa = await _channel.invokeMethod('getIDFA');
    return idfa;
  }

  /*--------------------------------------------CheckData-----------------------------------------------------*/

  // "getPackageName" -
  // "getGPSLongitude"
  // "getGPSLatitude" -
  // "getOSType" ->
  // "getNetType" ->
  // "getAppList" ->
  // "getActiveList" ->
  // "getSdkVersion" ->
  // "getOSVersion" ->
  // "getPhoneBrand" ->
  // "getPhoneModel" ->
  // "getPhoneElectric"
  // "getPhoneIp" ->
  // "getNetCarrier" ->
  // "getDevelopType" -

  //上报时间 time
  static Future<String> get time async {
    final String packageName = await _channel.invokeMethod('time');
    return packageName;
  }

  //包名package_name
  static Future<String> get packageName async {
    final String packageName = await _channel.invokeMethod('getPackageName');
    return packageName;
  }

  //App名称
  static Future<String> get appName async {
    final String appName = await _channel.invokeMethod('getAppName');
    return appName;
  }

  //经度 gps_longitude
  static Future<String> get gpsLongitude async {
    final String getGPSLongitude =
        await _channel.invokeMethod('getGPSLongitude');
    return getGPSLongitude;
  }

  //纬度 gps_latitude
  static Future<String> get gpsLatitude async {
    final String getGPSLatitude = await _channel.invokeMethod('getGPSLatitude');
    return getGPSLatitude;
  }

  //操作系统标识 os_type
  static Future<String> get osType async {
    final String getOSType = await _channel.invokeMethod('getOSType');
    return getOSType;
  }

  //网络类型 net_type
  static Future<String> get netType async {
    final String getNetType = await _channel.invokeMethod('getNetType');
    return getNetType;
  }

  //app列表 app_list
  static Future<String> get appList async {
    final String getAppList = await _channel.invokeMethod('getAppList');
    return getAppList;
  }

  //app活跃列表 app_active_list
  static Future<String> get appActiveList async {
    final String getActiveList = await _channel.invokeMethod('getActiveList');
    return getActiveList;
  }

  //sdk版本 sdk_version
  // static Future<String> get sdkVersion async {
  //   final String packageName = await _channel.invokeMethod('getSdkVersion');
  //   return packageName;
  // }

  //操作系统版本 os_version
  static Future<String> get osVersion async {
    final String getOSVersion = await _channel.invokeMethod('getOSVersion');
    return getOSVersion;
  }

  //手机品牌 phone_brand
  static Future<String> get phoneBrand async {
    final String getPhoneBrand = await _channel.invokeMethod('getPhoneBrand');
    return getPhoneBrand;
  }

  //手机型号 phone_model
  static Future<String> get phoneModel async {
    final String getPhoneModel = await _channel.invokeMethod('getPhoneModel');
    return getPhoneModel;
  }

  // 手机电量	phone_electric
  static Future<String> get phoneElectric async {
    final String getPhoneElectric =
        await _channel.invokeMethod('getPhoneElectric');
    return getPhoneElectric;
  }

  // 手机IPV4地址	phone_ip
  static Future<String> get phoneIp async {
    final String getPhoneIp = await _channel.invokeMethod('getPhoneIp');
    return getPhoneIp;
  }

  // 网络运营商	net_carrier
  static Future<String> get netCarrier async {
    final String getNetCarrier = await _channel.invokeMethod('getNetCarrier');
    return getNetCarrier;
  }

  // Oaid
  static Future<String> get oaid async {
    final String oaid = await _channel.invokeMethod('getOaid');
    return oaid;
  }

  // Amass——MD
  static Future<String> getAmassMD(
      String appKey, String unCollect, String sdkVersion) async {
    final String amassMd =
        await _channel.invokeMethod('getAmassMd', <String, dynamic>{
      "appKey": appKey,
      "unCollect": unCollect,
      "sdkVersion": sdkVersion,
    });
    return amassMd;
  }

  // develop_type	开发方式	是	是	是
  // 分为原生和flutter，
  // 原生：pro；
  // flutter：flutter
  static String get developType {
    String packageName = "flutter";
    return packageName;
  }

  // Crash——MD
  static Future<String> getCrashMd(
      String appKey, String sdkVersion, String crashLog, String dataType, String crashType) async {
    final String crashMd =
    await _channel.invokeMethod('getCrashData', <String, dynamic>{
      "appKey": appKey,
      "sdkVersion": sdkVersion,
      "crashLog": crashLog,
      "dataType": dataType,
      "crashType": crashType,
    });
    return crashMd;
  }
}
